-------------------------------------------------------------------------------
--
-- ClearOS Firewall
--
-------------------------------------------------------------------------------
--
-- Original by Dinesh Kandiah
-- + additions by Daniel Carrera
-- + additions by Paul Moore, pcmoore@engin.umich.edu
-- + port to app-firewall (Lua) by darryl@pointclark.net
--
-- Copyright (C) 2000-2009 Point Clark Networks
-- Copyright (C) 2003 Paul Moore
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- Inspired by...
-- + Trinity OS
-- + Linux Security
-- + linux-firewall-tools.com
-- + Usenet
--
-------------------------------------------------------------------------------
--
-- Notes:
-- This script is run by app-firewall which is a combination of iptables, the
-- Lua interpreter and various custom functions.  The purpose of the port to
-- Lua is performance.  All iptables operations are queued and are not commited
-- until the end only if all input was valid.  If there are any errors then the
-- firewall will not be touched.  See /sbin/app-firewall -h for help and
-- /etc/rc.d/rc.firewall for an example of how to invoke this script.
--
-- Semantics:
-- + Incoming - packets destined for *this* machine
-- + Outgoing - packets from the LAN/DMZ to the outside world
-- + Forward  - packets from the outside world to the LAN/DMZ
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--
-- F U N C T I O N S
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--
-- Explode
--
-- Explodes a delimited string into a Lua table.
--
-------------------------------------------------------------------------------

function Explode(d, s)
	local i = 0
	local p = 0
	local l = 0
	local t = {}

	while true do
		i = string.find(s, d, i + 1)

		if i ~= nil then
			l = i - 1
		else
			l = string.len(s)
		end

		table.insert(t, string.sub(s, p, l))
		if i == nil then break end

		p = i + 1
	end

	return t
end

-------------------------------------------------------------------------------
--
-- PackWhitespace
--
-- Removes redundant white-space from supplied string.
--
-------------------------------------------------------------------------------

function PackWhitespace(s)
	local c = 1

	while c > 0 do
		s, c = string.gsub(s, "  ", " ")
	end

	c = 1
	while c > 0 do
		s, c = string.gsub(s, "\t\t", " ")
	end

	c = 1
	while c > 0 do
		s, c = string.gsub(s, "\n\n", " ")
	end

	return s
end

-------------------------------------------------------------------------------
--
-- TableCount
--
-- Returns the number of items in a dictionary table (where getn doesn't work).
--
-------------------------------------------------------------------------------

function TableCount(t)
	local k
	local v
	local c = 0
	for k, v in pairs(t) do c = c + 1 end
	return c
end

-------------------------------------------------------------------------------
--
-- TablePrint
--
-- Print contents of a dictionary table to assist debugging.
--
-------------------------------------------------------------------------------

function TablePrint(t)
	local k
	local v
	local c = 0
	for k, v in pairs(t) do
		debug(string.format("%4d: %20s => %s", c, k, v))
		c = c + 1
	end
end

function ipxtables(table, rule)
	if IP_VER == "IPV4" then
		iptables(table, rule)
	elseif IP_VER == "IPV6" then
		ip6tables(table, rule)
	end
end

function StripIpMask(ip)
	_ = string.find(ip, "/")
	if _ == nil then
		return ip
	else
		return string.sub(ip, 0, _ - 1)
	end
end

function DisableRule(r_type, r_proto, r_addr, r_port, r_param)
	local f = io.open("/var/lib/firewall/invalid.state", "a+")
	f:write(string.format("||0x%08x|%d|%s|%s|%s\n", r_type, r_proto, r_addr, r_port, r_param))
	io.close(f)

	r_type = r_type - tonumber(os.getenv("FWR_ENABLED"))
	return string.format("||0x%08x|%d|%s|%s|%s", r_type, r_proto, r_addr, r_port, r_param)
end

-------------------------------------------------------------------------------
--
-- ValidateRule
--
-- Perform a series of sanity checks (address resolution) on a rule.  If the
-- rule is found to be invalid, the rule is temporary disabled and is also
-- added to an invalid rule state file (for other programs to see).
--
-------------------------------------------------------------------------------

function ValidateRule(r)
	local r_type
	local r_proto
	local r_addr
	local r_port
	local r_param
	local ip1
	local ip2
	local network
	local prefix
	local ifn_wan = nil

	r_type, r_proto, r_addr, r_port, r_param = ExpandRule(r)

	if b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) == 0 then
		return r
	end

	if (b_and(r_type, tonumber(os.getenv("FWR_INCOMING_BLOCK"))) ~= 0 or
		b_and(r_type, tonumber(os.getenv("FWR_DMZ_INCOMING"))) ~= 0 or
		b_and(r_type, tonumber(os.getenv("FWR_DMZ_PINHOLE"))) ~= 0 or
		b_and(r_type, tonumber(os.getenv("FWR_PROXY_BYPASS"))) ~= 0 or
		b_and(r_type, tonumber(os.getenv("FWR_FORWARD"))) ~= 0) and
		b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) == 0 then

		ip1 = gethostbyname(StripIpMask(r_addr))

		if ip1 == nil then
			r = DisableRule(r_type, r_proto, r_addr, r_port, r_param)
		else
			r = string.format("||0x%08x|%d|%s|%s|%s", r_type, r_proto, r_addr, r_port, r_param)
		end
	elseif b_and(r_type, tonumber(os.getenv("FWR_OUTGOING_BLOCK"))) ~= 0 and
		b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) == 0 and
		r_addr ~= nil and string.len(r_addr) ~= 0 then

		ip1 = gethostbyname(StripIpMask(r_addr))

		if ip1 == nil then
			r = DisableRule(r_type, r_proto, r_addr, r_port, r_param)
		else
			r = string.format("||0x%08x|%d|%s|%s|%s", r_type, r_proto, r_addr, r_port, r_param)
		end
	elseif b_and(r_type, tonumber(os.getenv("FWR_ONE_TO_ONE"))) ~= 0 then

		ip1 = gethostbyname(r_addr)
		ip2 = gethostbyname(r_param)

		_ = string.find(r_param, "_")

		if _ == 1 then
			ip2 = string.sub(r_param, 2)
		elseif _ ~= nil then
			ifn_wan = string.sub(r_param, 1, _ - 1)
			ip2 = string.sub(r_param, _ + 1)
		end

		if ifn_wan ~= nil then ip2 = ifn_wan .. "_" .. ip2 end

		if ip1 == nil or ip2 == nil then
			r = DisableRule(r_type, r_proto, r_addr, r_port, r_param)
		else
			r = string.format("||0x%08x|%d|%s|%s|%s", r_type, r_proto, ip1, r_port, ip2)
		end
	end

	return r
end

-------------------------------------------------------------------------------
--
-- LoadEnvironment
--
-- Load and parse environment variables as Lua globals
--
-------------------------------------------------------------------------------

function LoadEnvironment()
	local i
	local f
	local t
	local ifn
	local rule
	local rules = {}

	echo("Loading environment")

	TABLES = { "filter", "mangle", "nat" }

	-- Exported firewall configuration variables 
	FW_MODE = os.getenv("MODE")
	FW_ADHOC = os.getenv("FW_ADHOC")
	WANIF = os.getenv("EXTIF")
	LANIF = os.getenv("LANIF")
	HOTIF = os.getenv("HOTIF")
	DMZIF = os.getenv("DMZIF")
	WIFIF = os.getenv("WIFIF")
	IP_VER = os.getenv("IP_VER")
	BANDWIDTH_QOS = os.getenv("BANDWIDTH_QOS")
	BANDWIDTH_UPSTREAM = os.getenv("BANDWIDTH_UPSTREAM")
	BANDWIDTH_UPSTREAM_BURST = os.getenv("BANDWIDTH_UPSTREAM_BURST")
	BANDWIDTH_UPSTREAM_CBURST = os.getenv("BANDWIDTH_UPSTREAM_CBURST")
	BANDWIDTH_DOWNSTREAM = os.getenv("BANDWIDTH_DOWNSTREAM")
	BANDWIDTH_DOWNSTREAM_BURST = os.getenv("BANDWIDTH_DOWNSTREAM_BURST")
	BANDWIDTH_DOWNSTREAM_CBURST = os.getenv("BANDWIDTH_DOWNSTREAM_CBURST")
	IPSEC_SERVER = os.getenv("IPSEC_SERVER")
	PPTP_SERVER = os.getenv("PPTP_SERVER")
	ONE_TO_ONE_NAT_MODE = os.getenv("ONE_TO_ONE_NAT_MODE")
	MULTIPATH = os.getenv("MULTIPATH")
	MULTIPATH_WEIGHTS = os.getenv("MULTIPATH_WEIGHTS")
	MULTIPATH_SKIP_DOWN_WANIF = os.getenv("MULTIPATH_SKIP_DOWN_WANIF")
	GATEWAYDEV = os.getenv("GATEWAYDEV")
	FW_DROP = os.getenv("FW_DROP")
	FW_LOG_DROPS = os.getenv("FW_LOG_DROPS")
	FW_ACCEPT = os.getenv("FW_ACCEPT")
	IPBIN = os.getenv("IPBIN")
	TCBIN = os.getenv("TCBIN")
	MODPROBE = os.getenv("MODPROBE")
	RMMOD = os.getenv("RMMOD")
	SYSCTL = os.getenv("SYSCTL")
	IFCONFIG = os.getenv("IFCONFIG")
	PPTP_PASSTHROUGH_FORCE = os.getenv("PPTP_PASSTHROUGH_FORCE")
	SYSWATCH_WANIF = os.getenv("SYSWATCH_WANIF")
	EGRESS_FILTERING = os.getenv("EGRESS_FILTERING")
	PROTOCOL_FILTERING = os.getenv("PROTOCOL_FILTERING")
	FLOATING_IPV4 = os.getenv("FLOATING_IPV4")
	FLOATING_IPV6 = os.getenv("FLOATING_IPV6")

	-- Validate variables
	if FW_MODE == nil then error("MODE not defined")
	else debug("FW_MODE=" .. FW_MODE) end

	if IP_VER == nil then error("IP_VER not defined")
	else debug("IP_VER=" .. IP_VER) end

	if WANIF == nil then
		error("WANIF not defined")
	else
		t = {}
		WANIF = Explode(" ", WANIF)
		for i, ifn in pairs(WANIF) do
			if ifn ~= nil and string.len(ifn) ~= 0 and string.find(ifn, ":") == nil then
				table.insert(t, ifn)
				debug("WANIF=" .. ifn)
			end
		end
		WANIF = t
	end

	if LANIF == nil then
		error("LANIF not defined")
	else
		t = {}
		LANIF = Explode(" ", LANIF)
		for i, ifn in pairs(LANIF) do
			if ifn ~= nil and string.len(ifn) ~= 0 and string.find(ifn, ":") == nil then
				table.insert(t, ifn)
				debug("LANIF=" .. ifn)
			end
		end
		LANIF = t
	end

	if HOTIF ~= nil then
		t = {}
		HOTIF = Explode(" ", HOTIF)
		for i, ifn in pairs(HOTIF) do
			if ifn ~= nil and string.len(ifn) ~= 0 and string.find(ifn, ":") == nil then
				table.insert(t, ifn)
				table.insert(LANIF, ifn)
				debug("HOTIF=" .. ifn)
			end
		end
		HOTIF = t
	else
		HOTIF = {}
	end

	if DMZIF == nil then
		error("DMZIF not defined")
	else
		t = {}
		DMZIF = Explode(" ", DMZIF)
		for i, ifn in pairs(DMZIF) do
			if ifn ~= nil and string.len(ifn) ~= 0 and string.find(ifn, ":") == nil then
				table.insert(t, ifn)
				debug("DMZIF=" .. ifn)
			end
		end
		DMZIF = t

		if FW_MODE == "gateway" and table.getn(DMZIF) ~= 0 then
			FW_MODE = "dmz"
			debug("Switching from gateway to DMZ mode, DMZ interface(s) detected")
		elseif FW_MODE == "dmz" and table.getn(DMZIF) == 0 then
			FW_MODE = "gateway"
			debug("Switching from DMZ mode to gateway mode, no DMZ interface(s) detected")
		end
	end

	if SYSWATCH_WANIF == nil then
		SYSWATCH_WANIF = {}
	else
		t = {}
		SYSWATCH_WANIF = Explode(" ", SYSWATCH_WANIF)
		for i, ifn in pairs(SYSWATCH_WANIF) do
			if ifn ~= nil and string.len(ifn) ~= 0 then
				table.insert(t, ifn)
				debug("SYSWATCH_WANIF=" .. ifn)
			end
		end
		SYSWATCH_WANIF = t
	end

	if WIFIF == nil then WIFIF="" end
	debug("WIFIF=" .. WIFIF)

	if BANDWIDTH_QOS == nil then BANDWIDTH_QOS="on" end
	debug("BANDWIDTH_QOS=" .. BANDWIDTH_QOS)

	if BANDWIDTH_UPSTREAM == nil then BANDWIDTH_UPSTREAM = 1000000000 end
	debug("BANDWIDTH_UPSTREAM=" .. BANDWIDTH_UPSTREAM)

	if BANDWIDTH_DOWNSTREAM == nil then BANDWIDTH_DOWNSTREAM = 1000000000 end
	debug("BANDWIDTH_DOWNSTREAM=" .. BANDWIDTH_DOWNSTREAM)

	if IPSEC_SERVER == nil then IPSEC_SERVER = "off" end
	debug("IPSEC_SERVER=" .. IPSEC_SERVER)

	if PPTP_SERVER == nil then PPTP_SERVER = "off" end
	debug("PPTP_SERVER=" .. PPTP_SERVER)

	if ONE_TO_ONE_NAT_MODE == nil then ONE_TO_ONE_NAT_MODE = "type2" end
	debug("ONE_TO_ONE_NAT_MODE=" .. ONE_TO_ONE_NAT_MODE)

	if MULTIPATH == nil then MULTIPATH = "off"
	elseif MULTIPATH == "yes" then MULTIPATH = "on" end

	if MULTIPATH == "on" then
		if MULTIPATH_WEIGHTS == nil or MULTIPATH_WEIGHTS == "" then
			MULTIPATH_WEIGHTS = {}
			for _, ifn in pairs(WANIF) do
				table.insert(MULTIPATH_WEIGHTS, ifn .. "|1")
			end
		else
			t = {}
			MULTIPATH_WEIGHTS = Explode(" ", MULTIPATH_WEIGHTS)
			for i, ifn in pairs(MULTIPATH_WEIGHTS) do
				if ifn ~= nil and string.len(ifn) ~= 0 then
					table.insert(t, ifn)
					debug("MULTIPATH_WEIGHTS=" .. ifn)
				end
			end
			MULTIPATH_WEIGHTS = t
		end
	end

	if MULTIPATH_SKIP_DOWN_WANIF ~= nil and MULTIPATH_SKIP_DOWN_WANIF ~= "off" then
		MULTIPATH_SKIP_DOWN_WANIF = nil
	end

	-- Empty invalid rule state file
	f = io.open("/var/lib/firewall/invalid.state", "w+")
	if f ~= nil then io.close(f) end

	-- Explode rules in to a table
	if IP_VER == "IPV4" or IP_VER == "IPV6" then
		RULES = os.getenv("RULES")
	else
		error("IP_VER is not IPV4 or IPV6")
	end

	if RULES ~= nil then
		rules = Explode(" ", string.gsub(PackWhitespace(RULES), "\t", ""))
	end

	-- Validate firewall rules
	RULES = {}
	for _, rule in pairs(rules) do
		if rule ~= nil and string.len(rule) ~= 0 then
			rule = ValidateRule(rule)
			table.insert(RULES, rule)
			debug("RULES=" .. rule)
		end
	end

	if FW_DROP == nil then FW_DROP = "DROP" end
	debug("FW_DROP=" .. FW_DROP)

	if FW_ACCEPT == nil then FW_ACCEPT = "ACCEPT" end
	debug("FW_ACCEPT=" .. FW_ACCEPT)

	if IPBIN == nil then IPBIN = "/sbin/ip" end
	debug("IPBIN=" .. IPBIN)

	if TCBIN == nil then TCBIN = "/sbin/tc" end
	debug("TCBIN=" .. TCBIN)

	if MODPROBE == nil then MODPROBE = "/sbin/modprobe" end
	debug("MODPROBE=" .. MODPROBE)

	if RMMOD == nil then RMMOD = "/sbin/rmmod" end
	debug("RMMOD=" .. RMMOD)

	if SYSCTL == nil then SYSCTL = "/sbin/sysctl" end
	debug("SYSCTL=" .. SYSCTL)

	if IFCONFIG == nil then IFCONFIG = "/sbin/ifconfig" end
	debug("IFCONFIG=" .. IFCONFIG)

	if PPTP_PASSTHROUGH_FORCE == nil then PPTP_PASSTHROUGH_FORCE = "no" end
	debug("PPTP_PASSTHROUGH_FORCE=" .. PPTP_PASSTHROUGH_FORCE)

	if EGRESS_FILTERING == nil then
		EGRESS_FILTERING = "off"
	elseif EGRESS_FILTERING ~= "on" and EGRESS_FILTERING ~= "off" then
		EGRESS_FILTERING = "off"
	end
	debug("EGRESS_FILTERING=" .. EGRESS_FILTERING)

	if PROTOCOL_FILTERING == nil then
		PROTOCOL_FILTERING = "off"
	elseif PROTOCOL_FILTERING ~= "on" and PROTOCOL_FILTERING ~= "off" then
		PROTOCOL_FILTERING = "off"
	end

	if PROTOCOL_FILTERING == "on" then
		if execute("/etc/init.d/l7-filter status >/dev/null 2>&1") ~= 0 then
			echo("WARNING: l7-filter is not running, disabling protocol filtering.")
			PROTOCOL_FILTERING = "off"
		end
	end
	debug("PROTOCOL_FILTERING=" .. PROTOCOL_FILTERING)
end

function SetICMPVersionDefinitions()
	if IP_VER == "IPV4" then
		ICMP_TYPE_ECHO_REPLY = "-p icmp --icmp-type 0"
		ICMP_TYPE_DEST_UNREACHABLE = "-p icmp --icmp-type 3"
		ICMP_TYPE_ECHO_REQUEST = "-p icmp --icmp-type 8"
		ICMP_TYPE_TIME_EXCEEDED = "-p icmp --icmp-type 11"
		ICMP_TYPE_NONE = "-p icmp"
	elseif IP_VER == "IPV6" then
		ICMP_TYPE_ECHO_REPLY = "-p icmpv6 --icmpv6-type 129"
		ICMP_TYPE_DEST_UNREACHABLE = "-p icmpv6 --icmpv6-type 1"
		ICMP_TYPE_ECHO_REQUEST = "-p icmpv6 --icmpv6-type 128"
		ICMP_TYPE_NEIGHBOR_SOLICITATION = "-p icmpv6 --icmpv6-type 135"
		ICMP_TYPE_NEIGHBOR_ADVERTISEMENT = "-p icmpv6 --icmpv6-type 136"
		ICMP_TYPE_TIME_EXCEEDED = "-p icmpv6 --icmpv6-type 3"
		ICMP_TYPE_NONE = "-p icmpv6"
	end
end

-------------------------------------------------------------------------------
--
-- GetInterfaceIPList
--
-- Returns interface's ip address list.
--
-------------------------------------------------------------------------------

function GetInterfaceIPList(ifn)
    local h = {}
    local t = {}
    local ip
    local ips = if_addresses(ifn)
    local floating_ips = nil
    local ip_fields

    if ips ~= nil then

    ips = Explode(",", ips)
        for _, ip in pairs(ips) do
            if ip ~= nil and string.len(ip) ~= 0 then
                h[ip] = true
            end
        end
    end

    if IP_VER == "IPV4" then
        floating_ips = FLOATING_IPV4
    elseif IP_VER == "IPV6" then
        floating_ips = FLOATING_IPV6
    end

    if floating_ips ~= nil then
        floating_ips = Explode(",", floating_ips)

        for _, ip in pairs(floating_ips) do
            if ip ~= nil and string.len(ip) ~= 0 then
                ip_fields = Explode("/", ip)
                if ip_fields[1] == ifn and ip_fields[2] ~= nil and string.len(ip_fields[2]) ~= 0 then
                    h[ ip_fields[2] ] = true
                end
            end
        end
    end

    for ip, _ in pairs(h) do
        table.insert(t, ip)
    end

    return t
end

-------------------------------------------------------------------------------
--
-- GetInterfaceGateway
--
-- Returns the gateway for the given network interface.  If none is found in
-- /etc/sysconfig/network-scripts/ifcfg-??? nil is returned.  PPPoE interfaces
-- are detected by /32 netmasks and if_dst_address() is used for their gateway.
-- DHCP interface gateways are found in: /var/lib/dhclient/*.routers
--
-------------------------------------------------------------------------------

function GetInterfaceGateway(ifn)
	local f
	local gw
	local netmask = if_netmask(ifn)

	-- PPPOEKLUDGE: Return the peer IP address for PPP interfaces 
	if(IP_VER == "IPV4" and netmask == "255.255.255.255") or
		(IP_VER == "IPV6" and netmask == "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff") then
		return if_dst_address(ifn)
	end

	f = io.open("/etc/sysconfig/network-scripts/ifcfg-" .. ifn)

	if f ~= nil then
		for line in f:lines() do
			_, _, gw = string.find(line, "GATEWAY%s*=%s*\"*(%d+\.%d+\.%d+\.%d+)\"*")
			if gw ~= nil then break end
		end

		io.close(f)
	end

	if gw ~= nil then return gw end

	-- DHCPKLUDGE: Return stowed gateway (router) for DHCP interfaces
	f = io.open("/var/lib/dhclient/" .. ifn .. ".routers")

	if f ~= nil then
		for line in f:lines() do
			_, _, gw = string.find(line, "(%d+\.%d+\.%d+\.%d+)")
			if gw ~= nil then break end
		end

		io.close(f)
	end

	return gw
end

-------------------------------------------------------------------------------
--
-- GetPhysicalInterface
--
-- Returns the bridge interface associated with the given PPP device.
--
-------------------------------------------------------------------------------

function GetPhysicalInterface(ifn)
	local f
	local dev

	f = io.open("/etc/sysconfig/network-scripts/ifcfg-" .. ifn)

	if f ~= nil then
		for line in f:lines() do
			_, _, dev = string.find(line, "ETH%s*=%s*\"*(%w+)\"*")
			if dev ~= nil then break end
		end

		io.close(f)
	end

	if dev == nil then dev = ifn end

	return dev
end

-------------------------------------------------------------------------------
--
-- GetUntrustedInterfaces
--
-- Returns a table of untrusted interfaces - DMZIF, HOTIF, WANIF
-- If all_interfaces is true, use WANIF_CONFIG instead of WANIF.
--
-------------------------------------------------------------------------------

function GetUntrustedInterfaces(all_interfaces)
	local ifn
	local ifn_trusted = {}

	for _, ifn in pairs(HOTIF) do
		table.insert(ifn_trusted, ifn)
	end

	for _, ifn in pairs(DMZIF) do
		table.insert(ifn_trusted, ifn)
	end

	if all_interfaces == true then
		for _, ifn in pairs(WANIF_CONFIG) do
			table.insert(ifn_trusted, ifn)
		end
	else
		for _, ifn in pairs(WANIF) do
			table.insert(ifn_trusted, ifn)
		end
	end

	return ifn_trusted
end

-------------------------------------------------------------------------------
--
-- GetTrustedInterfaces
--
-- Returns a table of LAN interfaces (not Hot LAN)
--
-------------------------------------------------------------------------------

function GetTrustedInterfaces()
	local ifn
	local ifn_trusted = {}
	local is_hot

	for _, ifn in pairs(LANIF) do
		is_hot = false

		for __, ifn_hot in pairs(HOTIF) do
			if ifn == ifn_hot then is_hot = true; break end 
		end

		if is_hot == false then
			table.insert(ifn_trusted, ifn)
		end
	end

	return ifn_trusted
end

-------------------------------------------------------------------------------
--
-- IsValidInterface
--
-- Determine if WAN interface is active/valid.
--
-------------------------------------------------------------------------------

function IsValidInterface(ifn)
	local ifn_wan

	if MULTIPATH_SKIP_DOWN_WANIF ~= nil then return true end

	for _, ifn_wan in pairs(WANIF) do
		if ifn == ifn_wan then return true end
	end

	return false
end

-------------------------------------------------------------------------------
--
-- NetworkInterfaces
--
-- The following interfaces are exported from the firewall configuration file:
-- WANIF     - WAN interface(s) (EXTIF in /etc/firewall)
-- LANIF     - LAN interface(s)
-- DMZIF     - DMZ interface(s)
-- WIFIF     - Wireless interface
--
-------------------------------------------------------------------------------

function NetworkInterfacesKludgeFallback()
	local i
	local f
	local ifn
	local ifn_ppp
	local ifn_pppoe = {}
	local ifn_wan
	local pppoe_list
	local difs = {}

	-- Build list of PPPoE bridge interfaces
	pppoe_list = if_list_pppoe()

	if pppoe_list ~= nil then
		table.sort(pppoe_list)
		for _, ifn in pairs(pppoe_list) do
			table.insert(ifn_pppoe, GetPhysicalInterface(ifn))
		end
	end

	-- All interfaces without a set role default to a LAN interface
	for _, ifn in pairs(SYSWATCH_WANIF) do
		if string.len(ifn) ~= 0 then table.insert(difs, ifn) end
	end

	for _, ifn in pairs(LANIF) do
		if string.len(ifn) ~= 0 then table.insert(difs, ifn) end
	end

	for _, ifn in pairs(DMZIF) do
		if string.len(ifn) ~= 0 then table.insert(difs, ifn) end
	end

	for i, ifn in pairs(if_list()) do
		if string.find(ifn, "^[ae]th%d$") then
			f = false
			for _, difn in pairs(difs) do
				if ifn == difn then f = true end
			end

			-- PPPOEKLUDGE... sigh.
			if not f then
				for __, ifn_ppp in pairs(ifn_pppoe) do
					if ifn == ifn_ppp then f = true; break end
				end

				if f then
					echo("Detected PPPoE bridge device: " .. ifn)
				else
					for ___, ifn_wan in pairs(WANIF) do
						if ifn == ifn_wan then f = true; break end
					end

					if f == false then
						echo("Assuming device is a LAN interface: " .. ifn)
						--table.insert(LANIF, ifn)
						table.insert(WANIF, ifn)
					end
				end
			end
		end
	end

end

function NetworkInterfaces()
    -- NetworkInterfacesKludgeFallback()

	WANIF_CONFIG = WANIF
	WANIF = SYSWATCH_WANIF

	-- Display detected interface roles
	for _, ifn in pairs(WANIF) do
		echo("Detected WAN role for interface: " .. ifn)
	end

	for _, ifn in pairs(LANIF) do
		echo("Detected LAN role for interface: " .. ifn)
	end

	for _, ifn in pairs(DMZIF) do
		echo("Detected DMZ role for interface: " .. ifn)
	end
end

-------------------------------------------------------------------------------
--
-- SanityChecks
--
-- + Make sure we have an IP address set on our WAN interface.
-- + Make sure LAN and DMZ interfaces are configured (network-scripts present).
-- + Make sure all configured interfaces are UP.
--
-------------------------------------------------------------------------------

function SanityChecks()
	local f
	local ifn
	local t = {}
	local rule
	local rules = {}
	local r_type
	local r_proto
	local r_addr
	local r_port
	local r_param
	local ips
	local ip
	local network
	local prefix

	-- Ensure address of WAN interfaces are set
	t = WANIF; WANIF = {}
	for _, ifn in pairs(t) do
		if if_addresses(ifn) == nil then
			echo("Failed to detect IP address for WAN interface: " .. ifn)
		else
			table.insert(WANIF, ifn)
		end
	end

	if table.getn(WANIF) == 0 then
		echo("WARNING: No configured WAN interfaces, continuing anyway...")
	end

	-- Firewall rules which contain the LOCAL_NETWORK or EXTERNAL_ADDR flags need
	-- to be expanded for each local network or external address.
	rules = RULES; RULES = {}
	for _, rule in pairs(rules) do
		r_type, r_proto, r_addr, r_port, r_param = ExpandRule(rule)
		if b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_LOCAL_NETWORK"))) ~= 0 then
		elseif b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_EXTERNAL_ADDR"))) ~= 0 then
			for __, ifn in pairs(WANIF) do
				ips = GetInterfaceIPList(ifn)
                if ip ~= nil then
                    for _, ip in pairs(ips) do
				        rule = string.format("||0x%08x|%d|%s|%s|%s",
					        r_type, r_proto, ip, r_port, r_param)
				        table.insert(RULES, rule)
		            end
                end
			end
		else
			table.insert(RULES, rule)
		end
	end
end

-------------------------------------------------------------------------------
--
-- ExpandRule
--
-- Return packed rule components; type, protocol, address, port, parameter
--
-------------------------------------------------------------------------------

function ExpandRule(r)
	local rule = Explode("|", r)
	return tonumber(rule[3]), tonumber(rule[4]), rule[5], rule[6], rule[7]
end

-------------------------------------------------------------------------------
--
-- LoadLayer7Config
--
-- Loads/parses the l7-filter configuration file
--
-------------------------------------------------------------------------------

function LoadLayer7Config()
	local line
	local config = {}
	local f = io.open("/etc/l7-filter/l7-filter.conf", "r")

	if f ~= nil then
		for line in f:lines() do
			_, _, protocol, mark = string.find(line, "^([%w-]+)%s+(%d+)")
			if protocol ~= nil and mark ~= nil then
				config[protocol] = mark
			end
		end

		io.close(f)
	end

	return config
end

-------------------------------------------------------------------------------
--
-- SetKernelSettings
-- 
-- Defines some default kernel settings...  mostly for added security.
--
-------------------------------------------------------------------------------

function SetKernelSettings()
	echo("Setting kernel parameters")

	-- Enable IP Forwarding, not really required for standalone mode
	execute(SYSCTL .. " -w net.ipv4.ip_forward=1 >/dev/null")

	-- Enable TCP SYN Cookie protection
	execute(SYSCTL .. " -w net.ipv4.tcp_syncookies=1 >/dev/null")

	-- TODO: What was this?  Duplicate syncookie sysctl disabled.
	-- Enable dynamic TCP/IP address hacking
	-- execute(SYSCTL .. " -w net.ipv4.tcp_syncookies=1 >/dev/null")

	-- Don't log spoofed, source-routed, or redirect packets
	execute(SYSCTL .. " -w net.ipv4.conf.all.log_martians=0 >/dev/null")

	-- Disable ICMP redirects
	execute(SYSCTL .. " -w net.ipv4.conf.all.accept_redirects=0 >/dev/null")
	execute(SYSCTL .. " -w net.ipv4.conf.all.send_redirects=0 >/dev/null")

	-- Ensure that source-routed packets are dropped
	execute(SYSCTL .. " -w net.ipv4.conf.all.accept_source_route=0 >/dev/null")

	-- Disable ICMP broadcast echo protection
	execute(SYSCTL .. " -w net.ipv4.icmp_echo_ignore_broadcasts=1 >/dev/null")

	-- Enable bad error message protection
	execute(SYSCTL .. " -w net.ipv4.icmp_ignore_bogus_error_responses=1 >/dev/null")
end

-------------------------------------------------------------------------------
--
-- ClearRulesAndPolicies
-- 
-- Clears policies and policies
--
-------------------------------------------------------------------------------

function ClearRulesAndPolicies()
	local t

	echo("Clearing rules...")

	for _, t in pairs(TABLES) do
		-- Flush all previous rules
		iptc_flush_all_chains(t)
		-- Delete user-defined chains
		iptc_delete_user_chains(t)
	end

	iptc_set_policy("filter", "INPUT", FW_ACCEPT)
	iptc_set_policy("filter", "OUTPUT", FW_ACCEPT)
	iptc_set_policy("filter", "FORWARD", FW_ACCEPT)
end

-------------------------------------------------------------------------------
--
-- SetPolicyToDrop
-- 
-- Sets default firewall policy to DROP
--
-------------------------------------------------------------------------------

function SetPolicyToDrop()
	local t

	echo("Setting default policy to " .. FW_DROP)

	iptc_set_policy("filter", "INPUT", FW_DROP)
	iptc_set_policy("filter", "OUTPUT", FW_DROP)
	iptc_set_policy("filter", "FORWARD", FW_DROP)
end

-------------------------------------------------------------------------------
--
-- DefineChains
-- 
-- Define any custom chains here.  Custom chains include:
-- + drop-lan      - for logging LAN traffic trying to escape the LAN
--
-- Use the FW_DROP and FW_ACCEPT variable to override DROP and ACCEPT.  This
-- can be handy for trouble-shooting.
--
-------------------------------------------------------------------------------

function DefineChains()
	echo("Defining custom chains")

	if FW_LOG_DROPS == "yes" then
		FW_DROP = "DROP-log"
	end

	-- Create a default DROP chain for debugging
	if FW_DROP ~= "DROP" then
		for _, t in pairs(TABLES) do
			iptc_create_chain(t, FW_DROP)
			if FW_LOG_DROPS == "yes" then
				ipxtables(t, "-A " .. FW_DROP .. " -j LOG --log-prefix \"Drop: \"")
			end
			ipxtables(t, "-A " .. FW_DROP .. " -j DROP")
		end
	end

	-- Create a default ACCEPT chain for debugging
	if FW_ACCEPT ~= "ACCEPT" then
		iptc_create_chain("filter", FW_ACCEPT)
		-- ipxtables("filter", "-A " .. FW_ACCEPT .. " -j LOG --log-prefix \"Accept: \"")
		ipxtables("filter", "-A " .. FW_ACCEPT .. " -j ACCEPT")
	end

	-- Create a chain for dropping services that shouldn't leave the LAN
	iptc_create_chain("filter", "drop-lan")
	-- ipxtables("filter", "-A drop-lan -j LOG --log-prefix \"Drop - LAN only: \"")
	ipxtables("filter", "-A drop-lan -j DROP")
end

-------------------------------------------------------------------------------
--
-- LoadKernelModules
-- 
-- Loads kernel modules.  Most modules will automatically load... but some
-- require a little help :-)
--
-------------------------------------------------------------------------------

function LoadKernelModules()
	local modules = {}

	echo("Loading kernel modules")

	-- Add LOG target
	table.insert(modules, "ipt_LOG")
	-- Add REJECT target
	table.insert(modules, "ipt_REJECT")
	-- Connection tracking for FTP
	table.insert(modules, "ip_conntrack_ftp")
	-- Connection tracking for IRC
	table.insert(modules, "ip_conntrack_irc")
	-- PPTP and dependencies
	table.insert(modules, "ppp_generic")
	table.insert(modules, "ppp_mppe")
	table.insert(modules, "ip_conntrack_proto_gre")
	table.insert(modules, "ip_conntrack_pptp")
	-- IMQ for bandwidth QoS
	if BANDWIDTH_QOS == "on" then
		table.insert(modules, "imq")
		table.insert(modules, "ipt_IMQ")
	end

	for _, m in pairs(modules) do
		execute(string.format("%s %s >/dev/null 2>&1", MODPROBE, m))
	end
end

-------------------------------------------------------------------------------
--
-- UnloadNatKernelModules
-- 
-- Unloads kernel modules required for NAT
--
-------------------------------------------------------------------------------

function UnloadNatKernelModules()
	local modules = {}

	echo("Unloading unecessary NAT kernel modules")

	table.insert(modules, "ip_nat_ftp")
	table.insert(modules, "ip_nat_irc")
	table.insert(modules, "ip_nat_pptp")
	table.insert(modules, "ip_nat_h323")
	table.insert(modules, "ip_nat_proto_gre")
	table.insert(modules, "ipt_MASQUERADE")
	table.insert(modules, "iptable_nat")

	for _, m in pairs(modules) do
		execute(string.format("%s %s >/dev/null 2>&1", RMMOD, m))
	end
end

-------------------------------------------------------------------------------
--
-- RunCommonRules
--
-- Rules that should be included in *all* firewall modes should go here.
--
-- This function:
-- + allows all traffic on the loopback (localhost) interface
-- + blocks invalid address ranges
-- + allows ICMP (RFC compliance)
-- + allows DHCP traffic
--
-------------------------------------------------------------------------------

function RunCommonRules()
	local ip
	local netmask
	local network
	local prefix
	local ifn
	local ifn_hot
	local is_hot
	local global_broadaddr

	echo("Running common rules")

	-- SYN bit issues
	ipxtables("filter", "-A INPUT -m state --state INVALID -j DROP")
	ipxtables("filter",
		"-A INPUT -p tcp --tcp-flags SYN,ACK SYN,ACK -m state --state NEW -j REJECT --reject-with tcp-reset")
	ipxtables("filter", "-A INPUT -p tcp ! --syn -m state --state NEW -j DROP")

	if IP_VER == "IPV4" then
		-- Block addresses that should never show up on our WAN interface
		for _, ifn in pairs(WANIF_CONFIG) do
			ipxtables("filter", "-A INPUT -i " .. ifn .. " -s 127.0.0.0/8 -j DROP")
			ipxtables("filter", "-A INPUT -i " .. ifn .. " -s 169.254.0.0/16 -j DROP")
		end
	end

	-- Allow everything on the loopback interface
	ipxtables("filter", "-A INPUT -i lo -j " .. FW_ACCEPT)
	ipxtables("filter", "-A OUTPUT -o lo -j " .. FW_ACCEPT)

	-- Allow everything VPN interfaces
	ipxtables("filter", "-A INPUT -i pptp+ -j " .. FW_ACCEPT)
	ipxtables("filter", "-A OUTPUT -o pptp+ -j " .. FW_ACCEPT)
	ipxtables("filter", "-A INPUT -i tun+ -j " .. FW_ACCEPT)
	ipxtables("filter", "-A OUTPUT -o tun+ -j " .. FW_ACCEPT)

	-- Allow everything trusted LAN interfaces
	for _, ifn in pairs(LANIF) do
		is_hot = false

		for __, ifn_hot in pairs(HOTIF) do
			if ifn == ifn_hot then is_hot = true; break end 
		end

		if is_hot == false then
			ipxtables("filter", "-A INPUT -i " .. ifn .. " -j " .. FW_ACCEPT)
			ipxtables("filter", "-A OUTPUT -o " .. ifn .. " -j " .. FW_ACCEPT)
		end
	end

	if IP_VER == "IPV4" then
		global_broadaddr = "255.255.255.255"
	elseif IP_VER == "IPV6" then
		global_broadaddr = "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"
	end

	for _, ifn in pairs(WANIF_CONFIG) do
		-- Allow some ICMP (ping)
		--
		-- ICMP can be used for attacks.  We allow as little as possible.
		-- The following are necessary ports we *can't* do without:
		-- 0  Needed to ping hosts outside our network
		-- 3  Needed by all networks
		-- 8  Needed to ping this host (yes, it is an RFC requirement)
		-- 11 Needed by the traceroute application
		ipxtables("filter", string.format("-A INPUT -i %s %s -j %s", ifn, ICMP_TYPE_ECHO_REPLY, FW_ACCEPT))
		ipxtables("filter", string.format("-A INPUT -i %s %s -j %s", ifn, ICMP_TYPE_DEST_UNREACHABLE, FW_ACCEPT))
		ipxtables("filter", string.format("-A INPUT -i %s %s -j %s", ifn, ICMP_TYPE_ECHO_REQUEST, FW_ACCEPT))
		ipxtables("filter", string.format("-A INPUT -i %s %s -j %s", ifn, ICMP_TYPE_TIME_EXCEEDED, FW_ACCEPT))
		ipxtables("filter", string.format("-A OUTPUT -o %s %s -j %s", ifn, ICMP_TYPE_NONE, FW_ACCEPT))
		if IP_VER == "IPV6" then
			ipxtables("filter", string.format("-A INPUT -i %s %s -j %s", ifn, ICMP_TYPE_NEIGHBOR_SOLICITATION, FW_ACCEPT))
			ipxtables("filter", string.format("-A INPUT -i %s %s -j %s", ifn, ICMP_TYPE_NEIGHBOR_ADVERTISEMENT, FW_ACCEPT))
		end

		-- Allow DHCP client to respond
		ipxtables("filter", "-A INPUT -i " .. ifn .. " -p udp --dport bootpc --sport bootps -j " .. FW_ACCEPT)
		ipxtables("filter", "-A INPUT -i " .. ifn .. " -p tcp --dport bootpc --sport bootps -j " .. FW_ACCEPT)
		ipxtables("filter", "-A OUTPUT -o " .. ifn .. " -p udp --sport bootpc --dport bootps -j " .. FW_ACCEPT)
		ipxtables("filter", "-A OUTPUT -o " .. ifn .. " -p tcp --sport bootpc --dport bootps -j " .. FW_ACCEPT)
	end
end

-------------------------------------------------------------------------------
--
-- RunIncomingAllowedDefaults
--
-------------------------------------------------------------------------------

function RunIncomingAllowedDefaults()
	local ifn

	echo("Running default incoming allowed rules")

	-- Allow all outbound and limited inbound on untrusted interfaces
	for _, ifn in pairs(GetUntrustedInterfaces(true)) do
		ipxtables("filter", "-A OUTPUT -o " .. ifn .. " -j " .. FW_ACCEPT)
		ipxtables("filter", "-A INPUT -i " .. ifn ..
			" -p udp --dport 1024:65535 -m state --state ESTABLISHED,RELATED -j " .. FW_ACCEPT)
		ipxtables("filter", "-A INPUT -i " .. ifn ..
			" -p tcp --dport 1024:65535 -m state --state ESTABLISHED,RELATED -j " .. FW_ACCEPT)
	end
end

-------------------------------------------------------------------------------
--
-- RunIncomingAllowed
--
-------------------------------------------------------------------------------

function RunIncomingAllowed()
	local r_type
	local r_proto
	local r_addr
	local r_port
	local r_param
	local input
	local output
	local ips
	local ip
	local netmask
	local network
	local prefix
	local rule
	local ifn
	local target_name
	local filter_addr

	echo("Running user-defined incoming rules")

	-- Incoming ports and port ranges
	for _, rule in pairs(RULES) do
		r_type, r_proto, r_addr, r_port, r_param = ExpandRule(rule)

		if b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_INCOMING_ALLOW"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_CUSTOM"))) == 0 then

			-- filter addresses, only keep current address family on final list
			filter_addr = {}
			if r_addr ~= "" then
				for _, ip in pairs(Explode(",", r_addr)) do
					if (IP_VER == "IPV4" and string.find(ip, ":") == nil) or (IP_VER == "IPV6" and string.find(ip, ".") == nil) then
						table.insert(filter_addr, ip)
					end
				end
			end

			if table.getn(filter_addr) == 0 then
				echo("Allowing incoming " .. p_name(r_proto) .. " port/range " .. r_port)
				target_name = FW_ACCEPT
			else
				target_name = string.format("pa_%x_%d_%s", r_type, r_proto, string.gsub(r_port, "-", "_"))
				ipxtables("filter", string.format("-N %s", target_name))
				for _, ip in pairs(filter_addr) do
					echo("Allowing incoming " .. p_name(r_proto) .. " port/range " .. r_port .. " from " .. ip)
					ipxtables("filter", string.format("-A %s --source %s -j %s", target_name, ip, FW_ACCEPT))
				end
				ipxtables("filter", string.format("-A %s -j %s", target_name, FW_DROP))
			end

			for __, ifn in pairs(GetUntrustedInterfaces(false)) do
				ips = GetInterfaceIPList(ifn)
				if ips ~= nil then
					for __, ip in pairs(ips) do
						input = string.format("-A INPUT -p %d -d %s", r_proto, ip)
						output = string.format("-A OUTPUT -p %d -o %s -s %s", r_proto, ifn, ip)

						if r_addr == nil then
							input = input .. " -s " .. r_addr
							output = output .. " -d " .. r_addr
						end

						ipxtables("filter", input .. " --dport " .. r_port .. " -j " .. target_name)
						ipxtables("filter", output .. " --sport " .. r_port .. " -j " .. FW_ACCEPT)
					end
				end
			end
		end
	end
end

-------------------------------------------------------------------------------
--
-- RunBlockedHosts
--
-------------------------------------------------------------------------------

function RunBlockedHosts()
	local t
	local s = ""
	local ifn
	local rule
	local r_type
	local r_proto
	local r_addr
	local r_port
	local r_param

	echo("Running blocked external rules")

	-- Block host rules
	for _, rule in pairs(RULES) do
		r_type, r_proto, r_addr, r_port, r_param = ExpandRule(rule)

		if b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_INCOMING_BLOCK"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_WIFI"))) == 0 and
			b_and(r_type, tonumber(os.getenv("FWR_CUSTOM"))) == 0 and
			string.len(r_addr) ~= 0 then

			echo("Blocking external host: " .. r_addr)

			for __, ifn in pairs(WANIF) do
				ipxtables("filter", string.format("-A FORWARD -s %s -j %s", r_addr, FW_DROP))
				ipxtables("filter", string.format("-A INPUT -s %s -j %s", r_addr, FW_DROP))
				ipxtables("filter", string.format("-A OUTPUT -d %s -j %s", r_addr, FW_DROP))
			end
		end
	end
end

-------------------------------------------------------------------------------
--
-- RunIncomingDenied
--
-------------------------------------------------------------------------------

function RunIncomingDenied()
	local t
	local s = ""
	local ifn
	local rule
	local r_type
	local r_proto
	local r_addr
	local r_port
	local r_param
	local mac_filter

	echo("Running incoming denied rules")

	-- MAC filter rules
	if string.len(WIFIF) ~= 0 then
		echo("Checking for wireless MAC filtering on: " .. WIFIF)

		for _, rule in pairs(RULES) do
			r_type, r_proto, r_addr, r_port, r_param = ExpandRule(rule)

			if b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) ~= 0 and
				b_and(r_type, tonumber(os.getenv("FWR_MAC_FILTER"))) ~= 0 and
				b_and(r_type, tonumber(os.getenv("FWR_WIFI"))) ~= 0 and
				b_and(r_type, tonumber(os.getenv("FWR_CUSTOM"))) == 0 then

				mac_filter = "yes"

				echo("Adding wireless MAC filtering for: " .. WIFIF)
				ipxtables("nat",
					string.format("-A PREROUTING -i %s -m mac --mac-source %s -j %s",
					WIFIF, r_addr, FW_ACCEPT))
			end

			if mac_filter ~= nil then
				ipxtables("nat", "-A PREROUTING -i " .. WIFIF .. " -j " .. FW_DROP)
			end
		end
	end

	-- Block configured l7-filter protocols (entire network)
	if PROTOCOL_FILTERING == "on" then
		local protocol
		local mark
		local config = LoadLayer7Config()
		local drop_target = "l7-filter-drop"

		iptc_create_chain("mangle", drop_target)
		ipxtables("mangle", "-A POSTROUTING -j " .. drop_target)
		for protocol, mark in pairs(config) do
			echo("Blocking l7-filter protocol across entire network: " .. protocol)
			ipxtables("mangle", string.format("-A %s -m mark --mark %d -j %s",
				drop_target, mark, FW_DROP))
		end
	end

	-- TODO:
	-- Block configured l7-filter protocols (by host)
end


-------------------------------------------------------------------------------
--
-- RunPortForwardRules
--
-------------------------------------------------------------------------------

function RunPortForwardRules()
	local ifn
	local ifn_wan
	local rule
	local r_type
	local r_proto
	local r_addr
	local r_port
	local r_param
	local ips
	local ip
	local netmask
	local network
	local prefix
	local to
	local dport

	echo("Running user-defined port forward rules")

	-- Normal forward rules
	for _, rule in pairs(RULES) do
		r_type, r_proto, r_addr, r_port, r_param = ExpandRule(rule)
		r_partial_fwd = (string.find(r_param, "!") == 1)
		r_param = string.gsub(r_param, "^!", "")
		if b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_FORWARD"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_CUSTOM"))) == 0 then

			echo(string.format("Port forwarding %s %s to %s %s (proto=%d, partial=%s)",
				p_name(r_proto), r_param, r_addr, r_port, r_proto, r_partial_fwd and "yes" or "no"))

			if string.len(r_port) == 0 then
				to = r_addr
				dport = r_param
			else
				if IP_VER == "IPV4" then
					to = r_addr .. ":" .. r_port
				elseif IP_VER == "IPV6" then
					to = "[" .. r_addr .. "]:" .. r_port
				end
				dport = r_port
			end

			mport, _ = string.gsub(dport, "-", ":")

			ipxtables("filter",
				string.format("-A FORWARD -p %d -d %s --dport %s -j %s",
				r_proto, r_addr, mport, FW_ACCEPT))

			ipxtables("filter",
				string.format("-A FORWARD -p %d -s %s --sport %s -j %s",
				r_proto, r_addr, mport, FW_ACCEPT))

			local need_mapping = 0
			local dportbegin, dportend = 0, 0
			local sportbegin, sportend = 0, 0

			local fwd_chain = nil

			for __, ifn in pairs(WANIF) do
				ips = GetInterfaceIPList(ifn)
				echo(string.format("applying forward rules to interface %s", ifn))

				if ips ~= nil then
					for __, ip in pairs(ips) do
						if string.find(r_port, "-") == nil or string.find(r_param, ":") == nil then
							ipxtables("nat",
								string.format("-A PREROUTING -d %s -p %d --dport %s -j DNAT --to %s",
									ip, r_proto, r_param, to))


							if not r_partial_fwd then
								ipxtables("nat",
									string.format("-A POSTROUTING -o %s -d %s -p %d --dport %s -j SNAT --to %s",
										ifn, r_addr, r_proto, mport, ip))
							end
						else
							need_mapping = 1

							local dportdata = Explode("-", r_port, "-")
							local sportdata = Explode(":", r_param, ":")

							dportbegin, dportend = tonumber(dportdata[1]), tonumber(dportdata[2])
							sportbegin, sportend = tonumber(sportdata[1]), tonumber(sportdata[2])

							fwd_chain = string.format("FWD-%d-%d-%d", r_proto, sportbegin, sportend)

							pcall(function () ipxtables("nat", '-X ' .. fwd_chain) end)
							pcall(function () ipxtables("nat", '-N ' .. fwd_chain) end)

							ipxtables("nat",
								string.format("-A PREROUTING -d %s -p %d --dport %s -j %s",
								ip, r_proto, r_param, fwd_chain))

							if not r_partial_fwd then
								ipxtables("nat",
									string.format("-A POSTROUTING -o %s -d %s -p %d --dport %s -j SNAT --to %s",
									ifn, r_addr, r_proto, mport, ip))
							end

						end
					end
				end
			end

			if need_mapping == 1 then
				local i = 0
				while ((i + dportbegin) <= dportend) and ((i + sportbegin) <= sportend) do
					local temp_to
					if IP_VER == "IPV4" then
						temp_to = string.format("%s:%d", r_addr, dportbegin + i)
					elseif IP_VER == "IPV6" then
						temp_to = string.format("[%s]:%d", r_addr, dportbegin + i)
					end
					ipxtables("nat",
						string.format("-A %s -p %d --dport %d -j DNAT --to %s",
							fwd_chain, r_proto, sportbegin + i, temp_to))

					i = i + 1
				end

				if (i + dportbegin) <= dportend then
					echo(string.format('WARNING: destination port range larger than source port range by %d ports',
						(dportend - dportbegin) - i))
				end
				if (i + sportbegin) <= sportend then
					echo(string.format('WARNING: source port range bigger than destination port range by %d ports',
						(sportend - sportbegin) - i))
				end
			end
		end
	end
end

-------------------------------------------------------------------------------
--
-- RunCustomRules
--
-------------------------------------------------------------------------------

function RunCustomRules()
	local ips
	local ip
	local network
	local netmask
	local ifn
	local ifn_wan
	local rule
	local r_type
	local r_proto
	local r_addr
	local r_port
	local r_param
	local dst_addr
	local dst_port
	local input
	local output
	local forward
	local prerouting
	local postrouting

	echo("Running custom rules")

	for _, rule in pairs(RULES) do
		r_type, r_proto, r_addr, r_port, r_param = ExpandRule(rule)

		if b_and(r_type, tonumber(os.getenv("FWR_ENABLED"))) ~= 0 and
			b_and(r_type, tonumber(os.getenv("FWR_CUSTOM"))) ~= 0 then

			dst_addr = nil
			dst_port = nil

			if r_param ~= nil and string.len(r_param) ~= 0 then
				__ = string.find(r_param, "_")

				if __ == 1 then
					dst_port = string.sub(r_param, 2)
				elseif __ ~= nil then
					dst_addr = string.sub(r_param, 1, __ - 1)
					dst_port = string.sub(r_param, __ + 1)
				else
					dst_addr = r_param
				end
			end

			if b_and(r_type, tonumber(os.getenv("FWR_INCOMING_ALLOW"))) ~= 0 then
				-- INCOMING_ALLOW
				input = string.format("-A INPUT -p %s", r_proto)
				output = string.format("-A OUTPUT -p %s", r_proto)

				if r_addr ~= nil and string.len(r_addr) ~= 0 and
					b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) == 0 then
					input = input .. " -s " .. r_addr
					output = output .. " -d " .. r_addr
				elseif r_addr ~= nil and string.len(r_addr) ~= 0 and
					b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) ~= 0 then
					output = output .. " -m mac --mac-source " .. r_addr
				end

				if dst_addr ~= nil and string.len(dst_addr) ~= 0 then
					input = input .. " -d " .. dst_addr
					output = output .. " -s " .. dst_addr
				end

				if r_port ~= nil and string.len(r_port) ~= 0 and
					(tonumber(r_proto) == 6 or tonumber(r_proto) == 17) then
					input = input .. " --sport " .. r_port
					output = output .. " --dport " .. r_port
				end

				if dst_port ~= nil and string.len(dst_port) ~= 0 and
					(tonumber(r_proto) == 6 or tonumber(r_proto) == 17) then
					input = input .. " --dport " .. dst_port
					output = output .. " --sport " .. dst_port
				end

				for __, ifn in pairs(WANIF) do
					if string.find(ifn, ":") == nil then
						-- Only if ifn is a non-virtual interface
						if b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) == 0 then
							ipxtables("filter",
								string.format("%s -i %s -j %s", input, ifn, FW_ACCEPT))
						end
						ipxtables("filter",
							string.format("%s -o %s -j %s", output, ifn, FW_ACCEPT))
					end
				end
			elseif b_and(r_type, tonumber(os.getenv("FWR_INCOMING_BLOCK"))) ~= 0 then
				-- INCOMING_BLOCK
				input = string.format("-A INPUT -p %s", r_proto)
				output = string.format("-A OUTPUT -p %s", r_proto)

				if r_addr ~= nil and string.len(r_addr) ~= 0 and
					b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) == 0 then
					input = input .. " -s " .. r_addr
					output = output .. " -d " .. r_addr
				elseif r_addr ~= nil and string.len(r_addr) ~= 0 and
					b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) ~= 0 then
					input = input .. " -m mac --mac-source " .. r_addr
				end

				if dst_addr ~= nil and string.len(dst_addr) ~= 0 then
					input = input .. " -d " .. dst_addr
					output = output .. " -s " .. dst_addr
				end

				if r_port ~= nil and string.len(r_port) ~= 0 and
					(tonumber(r_proto) == 6 or tonumber(r_proto) == 17) then
					input = input .. " --dport " .. r_port
					output = output .. " --sport " .. r_port
				end

				if dst_port ~= nil and string.len(dst_port) ~= 0 and
					(tonumber(r_proto) == 6 or tonumber(r_proto) == 17) then
					input = input .. " --sport " .. dst_port
					output = output .. " --dport " .. dst_port
				end

				for __, ifn in pairs(WANIF) do
					if string.find(ifn, ":") == nil then
						-- Only if ifn is a non-virtual interface
						ipxtables("filter",
							string.format("%s -i %s -j %s", input, ifn, FW_DROP))
						if b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) == 0 then
							ipxtables("filter",
								string.format("%s -o %s -j %s", output, ifn, FW_DROP))
						end
					end
				end
			elseif b_and(r_type, tonumber(os.getenv("FWR_OUTGOING_BLOCK"))) ~= 0 then
				-- OUTGOING_BLOCK
				forward = "-A FORWARD -p " .. r_proto

				if r_addr ~= nil and string.len(r_addr) ~= 0 and
					b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) == 0 then
					forward = forward .. " -s " .. r_addr
				elseif r_addr ~= nil and string.len(r_addr) ~= 0 and
					b_and(r_type, tonumber(os.getenv("FWR_MAC_SOURCE"))) ~= 0 then
					forward = forward .. " -m mac --mac-source " .. r_addr
				end

				if dst_addr ~= nil and string.len(dst_addr) ~= 0 then
					forward = forward .. " -d " .. dst_addr
				end

				if r_port ~= nil and string.len(r_port) ~= 0 and
					(tonumber(r_proto) == 6 or tonumber(r_proto) == 17) then
					forward = forward .. " --sport " .. r_port
				end

				if dst_port ~= nil and string.len(dst_port) ~= 0 and
					(tonumber(r_proto) == 6 or tonumber(r_proto) == 17) then
					forward = forward .. " --dport " .. dst_port
				end

				ipxtables("filter", forward .. " -j " .. FW_DROP)
			elseif b_and(r_type, tonumber(os.getenv("FWR_FORWARD"))) ~= 0 then
				-- FORWARD
				prerouting = "-A PREROUTING -p " .. r_proto

				if dst_port ~= nil and string.len(dst_port) ~= 0 and
					(tonumber(r_proto) == 6 or tonumber(r_proto) == 17) then
					prerouting = prerouting .. " --dport " .. dst_port
				end

				if r_addr ~= nil and string.len(r_addr) ~= 0 then
					prerouting = prerouting .. " -s " .. r_addr
				end

				for __, ifn in pairs(WANIF) do
					ips = GetInterfaceIPList(ifn)
					if ips ~= nil then
						for __, ip in pairs(ips) do
							if dst_addr ~= nil and string.len(dst_addr) ~= 0 then
								ipxtables("nat",
									string.format("%s -d %s -j DNAT --to %s", prerouting, ip, dst_addr))
							else
								ipxtables("nat", string.format("%s -d %s", prerouting, ip))
							end
						end
					end
				end
			else
				error(string.format("Invalid custom firewall type: 0x%08x", r_type))
			end
		end
	end
end

-------------------------------------------------------------------------------
--
-- AddBandwidthRule
--
-------------------------------------------------------------------------------

function AddBandwidthClass(clsid, ifn, prio, rate, ceil, src_addr, src_port, dst_addr, dst_port)
	local src_text = ""
	local dst_text = ""
	local tc_filter = ""

	echo(string.format("HTB Class 1:%d, priority: %d, rate: %dkbit, ceil: %dkbit, interface: %s",
		clsid, prio, rate, ceil, ifn))

	-- Create class
	execute(string.format("%s class add dev %s parent 1:1 classid 1:%d htb rate %dkbit ceil %dkbit prio %d",
		TCBIN, ifn, clsid, rate, ceil, prio))

	-- Create tc filter for address and port
	if string.len(src_addr) ~= 0 then
		tc_filter = tc_filter .. " match ip src " .. src_addr
		src_text = src_addr
	end

	if string.len(dst_addr) ~= 0 then
		tc_filter = tc_filter .. " match ip dst " .. dst_addr
		dst_text = dst_addr
	end

	if string.len(src_port) ~= 0 and src_port ~= 0 then
		tc_filter = tc_filter .. " match ip sport " .. src_port .. " 0xffff"
		if string.len(src_text) ~= 0 then
			src_text = src_text .. ":" .. src_port
		else
			src_text = src_text .. src_port
		end
	end

	if string.len(dst_port) ~= 0 and dst_port ~= 0 then
		tc_filter = tc_filter .. " match ip dport " .. dst_port .. " 0xffff"
		if string.len(dst_text) ~= 0 then
			dst_text = dst_text .. ":" .. dst_port
		else
			dst_text = dst_text .. dst_port
		end
	end

	if string.len(tc_filter) == 0 then
		echo("Missing bandwidth match criteria")
	else
		if string.len(src_text) == 0 then
			src_text = "NONE"
		end
		if string.len(dst_text) == 0 then
			dst_text = "NONE"
		end
		echo(string.format("HTB Class 1:%d, source address: %s, destination address: %s",
			clsid, src_text, dst_text))

		-- Create tc filter
		execute(string.format("%s filter add dev %s protocol ip parent 1: " ..
			"pref 1 u32%s flowid 1:%d", TCBIN, ifn, tc_filter, clsid))
	end
end

-------------------------------------------------------------------------------
--
-- ParseBandwidthVariable
-- Parse BANDWIDTH_XXX firewall configuration value
-- This expects to find entries of the syntax: ifn:value, eg: eth0:512
--
-------------------------------------------------------------------------------

function ParseBandwidthVariable(v)
	local t = {}
	local entries = {}
	local cfg
	local ifn
	local value

	if v == nil or string.len(v) == 0 then return t end

	entries = Explode(" ", string.gsub(PackWhitespace(v), "\t", ""))

	for _, cfg in pairs(entries) do
		if cfg ~= nil and string.len(cfg) ~= 0 then
			__, __, ifn, value = string.find(cfg, "(%w+):(%d+)")

			if ifn == nil or value == nil then
				echo("Invalid bandwidth configuration syntax detected: " .. cfg)
				return nil
			end
			t[ifn] = value
		end
	end

	return t
end

-------------------------------------------------------------------------------
--
-- F I R E W A L L S
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--
-- S T A N D A L O N E
--
-- A "standalone" firewall is designed for a server that sits on the Internet
-- (or an untrusted LAN).  Allowed ports must be defined in: /etc/firewall
--
-------------------------------------------------------------------------------

function StandAlone()
	echo("Using standalone mode")

	UnloadNatKernelModules()
	LoadKernelModules()
	ClearRulesAndPolicies()
	DefineChains()
	RunBlockedHosts()
	RunCustomRules()
	RunCommonRules()
	RunIncomingDenied()
	RunIncomingAllowed()
	RunIncomingAllowedDefaults()
	RunPortForwardRules()
	-- Set policy now to avoid losing packets during reconfiguration
	SetPolicyToDrop()
end

-------------------------------------------------------------------------------
--
-- SetFirewallMode
--
-------------------------------------------------------------------------------

function SetFirewallMode()
	local ifn
	local ips
	local ip
	local netmask
	local network
	local prefix

	for _, ifn in pairs(WANIF) do
	    ips = GetInterfaceIPList(ifn)

		if ips ~= nil then
			for _, ip in pairs(ips) do
				echo(string.format("Detected WAN info - %s %s", ifn, ip))
			end
		end
		--echo(string.format("Detected WAN info - %s %s on network %s/%s",
		--	ifn, ip, network, prefix))
	end
end

-------------------------------------------------------------------------------
--
-- M A I N
--
-- Start the firewall
--
-------------------------------------------------------------------------------

echo("Starting firewall...")

iptc_init()

LoadEnvironment()
SetICMPVersionDefinitions()
NetworkInterfaces()
SanityChecks()
SetFirewallMode()
SetKernelSettings()

if FW_MODE == "standalone" then StandAlone()
else error("Invalid firewall mode: " .. FW_MODE) end

-- Commit changes
for _, t in pairs(TABLES) do iptc_commit(t) end

-- vi: syntax=lua ts=4
os.exit(0)
