#!/usr/bin/env python

# Copyright (C) 2014  Sangoma Technologies Corp.
# All Rights Reserved.
#
# Author(s):
# Leonardo Lang <lang@sangoma.com>

import re
import os
import sys
import logging
import subprocess
import tempfile

# this has to run on python 2.6
from contextlib import nested

logger = logging.getLogger()
logger_handler = logging.StreamHandler()
logger_formatter = logging.Formatter('%(levelname)s: %(message)s')
logger_handler.setFormatter(logger_formatter)
logger.addHandler(logger_handler)
logger.setLevel(logging.DEBUG if os.environ.get('DEBUG') else logging.INFO)

REGEX_KERNEL = re.compile('^(b?zImage|vmlinuz)-([^-]+-[^-]+)[.](.+)$')

YUM_CACHE_DIR = '/tmp/yum-empty-cache'
YUM_REPOS_DIR = '/tmp/yum-empty-repos'

YUM_CONFIG = \
"""[main]
cachedir={cache}
keepcache=0
debuglevel=2
logfile=/var/log/yum.log
exactarch=1
obsoletes=1
gpgcheck=1
plugins=1
installonly_limit=3
reposdir={repos}
""".format(cache=YUM_CACHE_DIR, repos=YUM_REPOS_DIR)

def run_pipe(args, data):
    logger.debug('running via pipes: {0!s}'.format(args))
    inpdata = data if isinstance(data, basestring) else '\n'.join(data)
    proc = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    outdata = proc.communicate(inpdata)[0]
    if proc.wait() != 0:
        logger.warning('process {0!s}[{1!s}] returned {2!s}'.format(args[0], proc.pid, proc.returncode))
    return outdata if isinstance(data, basestring) else outdata.splitlines()

def run(args, quiet=False):
    logger.debug('running: {0!s}'.format(args))
    with nested(open('/dev/null'), open('/dev/null', 'w')) as (nullrd, nullwr):
        kwargs = dict(stdin=nullrd) if not quiet else dict(stdin=nullrd, stdout=nullwr, stderr=nullwr)
        proc = subprocess.Popen(args, **kwargs)

        if proc.wait() != 0 and not quiet:
            logger.warning('process {0!s}[{1!s}] returned {2!s}'.format(args, proc.pid, proc.returncode))

        return proc.returncode

def main():
    retcode = 0

    kernel_versions = list()
    for name in os.listdir('/boot'):
        m = REGEX_KERNEL.match(name)
        if m is None:
            continue
        logger.info('found kernel {0}'.format(m.group(2)))
        kernel_versions.append(m.group(2))

    kernel_versions = run_pipe(['sort', '-r', '-V'], kernel_versions)

    try:
        latest_version, older_versions = kernel_versions[0], kernel_versions[1:]

    except IndexError:
        logger.info('no kernel currently installed, skipping cleanup procedure')
        return True

    logger.info('latest kernel is {0}'.format(latest_version))

    for yumdir in [ YUM_CACHE_DIR, YUM_REPOS_DIR ]:
        try:
            os.mkdir(yumdir)
        except:
            pass

    with tempfile.NamedTemporaryFile(prefix="yum", suffix=".conf") as yumconf:
        yumconf.write(YUM_CONFIG)
        yumconf.flush()

        for version in older_versions:
            logger.info('removing kernel {0}'.format(version))

            for prefix in [ 'kernel', 'kernel-ml' ]:
                pkgname = '{0}-{1}'.format(prefix, version)
                res = run(['rpm', '-q', pkgname], quiet=True)
                if res != 0:
                    continue

                run(['yum', '-c', yumconf.name, '-y', '-C', 'remove', pkgname])

                res = run(['rpm', '-q', pkgname], quiet=True)
                if res == 0:
                    logger.error('unable to remove kernel {0}'.format(version))
                    retcode = 1

            logger.info('scanning for letfover files for kernel {0}...'.format(version))

            pattern_string = '[-]{0}[.](x86_64|i?86)'.format(re.escape(version))
            pattern_regex = re.compile(pattern_string)

            for filename in os.listdir('/boot'):
                if pattern_regex.search(filename) is None:
                    continue

                logger.debug('moving {0} to /var/tmp...'.format(filename))
                if run(['mv', '-f', os.path.join('/boot', filename), os.path.join('/var/tmp/', filename)]) != 0:
                    retcode = 1

    initrd_regex = re.compile('^(initrd|initramfs)')

    pattern_string = '[-]{0}[.](x86_64|i?86)'.format(re.escape(latest_version))
    pattern_regex = re.compile(pattern_string)

    logger.info('scanning for ramdisks for past kernel versions...')

    for filename in os.listdir('/boot'):
        if initrd_regex.match(filename) is None:
            continue

        if pattern_regex.search(filename) is not None:
            continue

        logger.debug('moving {0} to /var/tmp...'.format(filename))
        if run(['mv', '-f', os.path.join('/boot', filename), os.path.join('/var/tmp/', filename)]) != 0:
            retcode = 1

    logger.info('cleanup procedure finished!')
    return retcode

sys.exit(main())
