#!/usr/bin/env python2.7
# vim: tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab
"""
* Copyright (C) 2011   Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* William Adam <william.adam@sangoma.com>
*
* This code is Sangoma Technologies Confidential Property.
* Use of and access to this code is covered by a previously executed
* non-disclosure agreement between Sangoma Technologies and the Recipient.
* This code is being supplied for evaluation purposes only and is not to be
* used for any other purpose.
"""

import requests
import json
import collections
import copy
import re
import os

from .object import Object
from .logger import logger as defaultlogger

DEFAULT_HOST = '127.0.0.1'
DEFAULT_PORT = 81
DEFAULT_PROTOCOL = 'http'

_SAFE_REST_URI = '/SAFe/sng_rest/'

class SafepyException(Exception):
    pass

def verify_socket_addr(value):
    '''
    Split and return strings of type <xxxx:yy> into tuple
    (xxxx, yy) with value checking
    '''
    if value is None or value is '':
        return None, None
    elif not isinstance(value, str):
        raise TypeError('Must pass a string!')

    if ':' in value:
        ip, port = value.split(':')
        if len(port) == 0:
            port = None
    else:  # either an ip OR port
        try: # value may be just port int
            int(value)
            port = value
            ip = None
        except ValueError:
            ip = value
            port = None

    import ipaddr

    if ip is not None:
        if ip == '0.0.0.0' or '*' in ip:
            raise ValueError("'" + ip + "' is not a valid ip address!")
        # run the address through ipaddr as a sanity check
        # and to catch any incorrect addresses
        ipaddr.IPv4Address(ip)

    if port == '*':
        raise ValueError("'" + port + "' is not a valid port address!")

    return ip, port

class Server(Object):
    def __init__(self, name, description=None, logger=None):
        self._uri = None
        self.logger = logger or defaultlogger
        super(Server, self).__init__(None, name, description)

    def server_socket(self, connection_string = None):
        if connection_string is None:
            # No server address specify, most likely trying to use local server
            # Try to find out the httpd listen directive
            if os.path.isfile('/usr/webconfig/conf/httpd.d/server_local.conf'):
                # Extract Listen argument
                r = re.compile("Listen\s+?([0-9.:].*)")
                m = r.search( 
                        open(
                            '/usr/webconfig/conf/httpd.d/server_local.conf',
                            "r").read())
                if m:
                    connection_string = m.group(1)

        return verify_socket_addr(connection_string)

    def connect(self,
            host=DEFAULT_HOST,
            port=DEFAULT_PORT,
            protocol=DEFAULT_PROTOCOL,
            **kwargs):
        # Account for None arguments
        if host is None:
            self.host = DEFAULT_HOST
        else:
            self.host = host

        if port is None:
            self.port = DEFAULT_PORT
        else:
            self.port = port

        if protocol is None:
            self.protocol = DEFAULT_PROTOCOL
        else:
            self.protocol = protocol

        # Build requests uri
        self._uri = self.protocol + '://' + self.host + ':' + str(self.port) + _SAFE_REST_URI
        self._api_uri = self._uri + 'api/'

        # Create a session
        self._session = requests.Session()
        self._adapter = requests.adapters.HTTPAdapter()
        self._session.mount(self.protocol + '://', self._adapter)

        # Custom headers
        self._session.headers = {'content-type': 'application/json'}

        self.logger.debug('connect: ' + self._uri)

    def discconnect(self):
        self.logger.debug('disconnect:' + self._uri)

    def _data_to_safe(self, data, sub_data=None, base_name=None):
        if not sub_data:
            sub_data = data

        if isinstance(sub_data, collections.Mapping):
            for k, v in sub_data.items():
                if base_name:
                    _name = base_name + '/' + k
                else:
                    _name = k

                if isinstance(v, collections.Mapping):
                    self._data_to_safe(data, v, _name)
                    del sub_data[k]
                else:
                    data[_name] = v

        return data

    # Create nested dict from list and adding value to last dict level
    def _nested_dict(self, d, l, v):
        _l = l.pop(0)
        if len(l):
            d[_l] = dict()
            self._nested_dict(d[_l], l, v)
        else:
            d[_l] = v

    # From http://www.xormedia.com/recursively-merge-dictionaries-in-python/
    def _merge_dict(self, a, b):
        '''recursively merges dict's. not just simple a['key'] = b['key'], if
        both a and bhave a key who's value is a dict then dict_merge is called
        on both values and the result stored in the returned dictionary.'''
        if not isinstance(b, dict):
            return b
        result = copy.deepcopy(a)
        for k, v in b.iteritems():
            if k in result and isinstance(result[k], dict):
                    result[k] = self._merge_dict(result[k], v)
            else:
                result[k] = copy.deepcopy(v)
        return result

    def _data_from_safe(self, data):
        if isinstance(data, collections.Mapping):
            for k, v in data.items():
                tokens = k.split('/')
                if len(tokens) > 1:
                    _data = dict()
                    self._nested_dict(_data, tokens, v)
                    data = self._merge_dict(data, _data)
                    del data[k]
                elif isinstance(v, collections.Mapping):
                    data[k] = self._data_from_safe(v)

        return data


    _last_error = {}
    @property
    def last_error(self):
        return self._last_error

    _last_response = {}
    @property
    def last_response(self):
        return self._last_response

    _last_request = {}
    @property
    def last_request(self):
        return self._last_request

    _raise_error = False
    @property
    def raise_error(self):
        return self._raise_error

    @raise_error.setter
    def raise_error(self, mode=False):
        self._raise_error = mode

    def _wrap_result(self, r):
        self._last_error = {}
        self._last_response = r.text
        self._last_request = r
        # Decore response
        try:
            response = json.loads(r.text,
                    object_pairs_hook=collections.OrderedDict)
            data = []
            if 200 == r.status_code:
                # Any data returned ?
                if 'data' in response:
                    data = self._data_from_safe(response['data'])
                else:
                    data = self._data_from_safe(response)
                return data
            else:
                # Any error returned ?
                if 'error' in response:
                    self._last_error['error'] = self._data_from_safe(response['error'])
        except ValueError:
            # Unable to json decode the response
            # If status_code == 200 then most likely an attachement (download)
            if 200 == r.status_code:
                data = {}
                # Store some usefull header values in data
                if 'content-length' in r.headers:
                    data['length'] = r.headers['content-length']
                if 'content-type' in r.headers:
                    data['type'] = re.sub('[\'"]', '', r.headers['content-type'])
                if 'content-transfer-encoding' in r.headers:
                    data['transfer-encoding'] = r.headers['content-transfer-encoding']

                # Store body content
                data['content'] = r.content

                # Check if 'content-disposition' header is present and contains 'attachment;' and 'filename='
                try:
                    # extract 'filename="xxx"'
                    data['filename'] = re.search('filename="(.+?)"',
                            r.headers['content-disposition']).group(1)
                except AttributeError:
                    data['filename'] = ''

                return data

        # Hitting this point means we didn't succeed to decode response
        self._last_error['status'] = r.status_code

        if self._raise_error:
            raise SafepyException(self._last_error)
        else:
            return False

    def do_post(self, uri, params=None, **kwargs):
        self.logger.debug('POST: ' + uri)
        if params:
            # Check if trying to post file
            if 'content' in params and 'filename' in params:
                saved_headers = self._session.headers
                self._session.headers = {}
                
                files = {'archive': (params['filename'], params['content'])}
                r = self._session.post(uri, files=files)
                self._session.headers= saved_headers
            else:
                p = copy.deepcopy(params)
                # do we have to convert params dict to flat / notation ?
                convert = kwargs.pop('convert', True)
                if convert:
                    p = self._data_to_safe(p)
                r = self._session.post(uri, data=json.dumps(p,
                    sort_keys=False))
        else:
            r = self._session.post(uri)
        return self._wrap_result(r)
        
    def do_get(self, uri, params=None, **kwargs):
        self.logger.debug('GET: ' + uri)
        if params:
            self.logger.debug('Redirect to POST due to params supplied: ' + uri)
            return self.do_post(uri, params, **kwargs)
        else:
            r = self._session.get(uri)
        return self._wrap_result(r)
        

    def delegate(self, request, method, path=None, params=None, category='api',
            **kwargs):
        if not self._uri:
            self.connect()

        _uri = self._uri + category
        if method:
            _uri = _uri + '/'  + method 
        if path:
            _uri = _uri + '/' + path
        # Call proper do_<request>
        action = getattr(self, 'do_' + request.lower())

        return action(_uri, params, **kwargs)

