#!/usr/bin/env python2.7
# vim: tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab
"""
* Copyright (C) 2015   Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* Leonardo Lang <lang@sangoma.com>
"""

import re
import sys
import subprocess
import socket
import select
import json
import logging
import logging.handlers
import time

import xml.dom.minidom as xml

from optparse import OptionParser

# Sample output:
#
# <program>
#     <name>Cluster Monitor</name>
#     <message>Output message</message>
# </program>
#
# rc =  0 => success
# rc = -1 => failure

MONITOR_SERVER_ADDRESS = ('127.0.0.1', 1010)

CLUSTER_SWITCH_NAME = "Cluster"
CLUSTER_GROUP_NAME = "NSC"
FLOATING_IP_PREFIX = "FloatingIP_"

def get_json_map(logger, res, *lst):
    try:
        return reduce(lambda res, elm: res[elm], lst, res)
    except KeyError as e:
        logger.debug('key "{name}" not found while acessing "{lst}"'.format(name=e.args[0], lst='.'.join(lst)))
        return dict()

def pretty_name(res_name):
    if res_name == CLUSTER_SWITCH_NAME:
        return 'NetBorder Session Controller';
    if res_name == CLUSTER_GROUP_NAME:
        return 'NSC group (all resources)'
    if res_name.startswith(FLOATING_IP_PREFIX):
        return 'Floating IP <' + res_name[len(FLOATING_IP_PREFIX):] + '>'
    return 'Resource <' + res_name + '>'

def load_monitor_status(logger):
    logger.debug('connecting to monitor status server..')
    sock = None
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect(MONITOR_SERVER_ADDRESS)
    except Exception as e:
        logger.debug('connection failed: {e}'.format(e=sys.exc_info()[1]))
        return None

    if len(select.select([], [sock.fileno()], [], 5)[1]) == 0:
        logger.error('unable to send data to cluster monitor: timeout waiting for socket')
        return None

    logger.debug('requesting status...')
    sock.send('STATUS\n')

    logger.debug('waiting for response...')
    buffer = ''
    while True:
        if len(select.select([sock.fileno()], [], [], 5)[0]) == 0:
            logger.error('unable to receive data from cluster monitor: timeout waiting for socket')
            return None

        data = None
        try:
            data = sock.recv(4096)
        except EnvironmentError, e:
            if e.errno not in [ errno.EAGAIN, errno.EINTR, errno.EWOULDBLOCK ]:
                continue
            raise
        if data == '':
            break
        buffer += data

    sock.close()
    return buffer

def get_monitor_status(logger, testfile):

    buffer = None
    if testfile is None:
        buffer = load_monitor_status(logger)
    else:
        fdes = open(testfile)
        buffer = fdes.read()
        fdes.close()

    json_data = None
    try:
        logger.debug('validating status configuration..')
        json_data = json.loads(buffer)
    except Exception as e:
        logger.error('got invalid data from cluster monitor: {e!s}'.format(e=sys.exc_info()[1]))
        return None

    result, failures = list(), dict()

    logger.debug('verifying resource status...')
    for node_name, node_data in get_json_map(logger, json_data, 'cluster', 'nodes').items():
        logger.debug('verifying resources for {0}...'.format(node_name))
        for res_name, res_data in get_json_map(logger, node_data, 'resources').items():

            for oper_data in get_json_map(logger, res_data, 'operations'):
                oper_name = oper_data.get('name', 'unknown')

                oper_rc = oper_data.get('rc', ['success'])
                oper_rc_name, oper_rc_desc = oper_rc[0], (oper_rc[1] if len(oper_rc) > 1 else 'unknown reason')

                if (oper_name not in [ 'start' ]) or (oper_rc_name == 'success'):
                    continue

                failures.setdefault(res_name, dict()).setdefault(node_name, dict()).setdefault('operations', list()).append((oper_name, oper_rc_desc))

            state_attr_map = get_json_map(logger, node_data, 'state-attributes')

            fail_count, last_failure = \
                state_attr_map.get('fail-count-' + res_name), \
                state_attr_map.get('last-failure-' + res_name)

            if fail_count is not None and last_failure is not None:
                last_failure = time.strftime("%F %T", time.localtime(float(last_failure)))
                failures.setdefault(res_name, dict()).setdefault(node_name, dict())['failures'] = \
                    '{fail_count} time{s}, last at {last_failure}'.format(s=('s' if fail_count > 1 else ''), **locals())

        for attr_name, attr_data in get_json_map(logger, node_data, 'attributes').items():
            if attr_name == 'standby' and attr_data == 'on':
                result.append('node "{node_name}" is disabled'.format(**locals()))

    for res_name, res_data in failures.items():
        messages = list()

        for node_name, node_data in res_data.items():
            oper_msg = ', '.join('{name} ({desc})'.format(name=e[0], desc=e[1]) for e in node_data.get('operations', list()))
            node_msg = '{msg} failed '.format(msg=oper_msg) if len(oper_msg) != 0 else 'execution failed '

            node_msg += 'on {node_name}'.format(**locals())

            fail_msg = node_data.get('failures')
            if fail_msg is not None:
                node_msg += ' ({fail_msg})'.format(**locals())

            messages.append(node_msg)

        result.append('resource {name} failed: {msgs}'.format(name=pretty_name(res_name),
            msgs=('' if len(messages) == 1 else '\n') + ';\n'.join(messages)))

    return result

def do_check(logger, options):
    rc, faulty = 0, list()

    status = get_monitor_status(logger, options.testfile)

    if status is None:
        message = 'Not running'
    else:
        if len(status) == 0:
            message = 'OK'
        else:
            message = ', '.join(status)
            rc = -1

    # Create output
    doc = xml.Document()

    root = doc.createElement('program');
    name = doc.createElement('name')
    text = doc.createTextNode("Cluster Monitor")
    name.appendChild(text)
    root.appendChild(name)

    mesg = doc.createElement('message')
    text = doc.createTextNode(message)
    mesg.appendChild(text)
    root.appendChild(mesg)

    # output result
    return rc, root


def main():
    parser = OptionParser()
    parser.add_option('-d', '--debug', action='store_true', default=False, help="enable debug mode")
    parser.add_option('-p', '--pretty', action='store_true', default=False, help="pretty print XML output")
    parser.add_option('-t', '--testfile', help="load status data from testfile")
    (options, args) = parser.parse_args()

    # Create logger
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG if options.debug else logging.INFO)
    logger.addHandler(logging.StreamHandler())

    # Execute Check
    rc, xmldata = do_check(logger, options)
    sys.stdout.write(xmldata.toprettyxml() if options.pretty else xmldata.toxml())
    sys.stdout.flush()
    return rc

if __name__ == '__main__':
    sys.exit(main())
