#!/usr/bin/env python2.7
# vim: tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab
"""
* Copyright (C) 2015   Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* Leonardo Lang <lang@sangoma.com>
"""

import sys
import re
import subprocess

import logging
import logging.handlers

import xml.dom.minidom as xml

from optparse import OptionParser

# Sample output:
#
# <program>
#     <name>Cluster Network Rings</name>
#     <message>Output message</message>
# </program>
#
# rc =  0 => success
# rc = -1 => failure

REGEX_CFGTOOL_STATUS = re.compile('^(Local node ID ([0-9]+))|^(RING ID ([0-9]+))|^([ \\t]+(id|status)[ \\t]+[=][ ](.+))')

def cfgtool_status(logger):
    proc = subprocess.Popen(['/usr/sbin/corosync-cfgtool', '-s'], stdout=subprocess.PIPE)
    outdata = proc.communicate()[0]

    if proc.returncode != 0:
        return None

    local_id, ring_id = None, None
    rings = dict()

    for line in outdata.splitlines():
        m = REGEX_CFGTOOL_STATUS.match(line)
        if not m:
            continue

        if m.group(2) is not None:
            local_id = m.group(2)
            continue
        if m.group(4) is not None:
            ring_id = m.group(4)
            rings[ring_id] = dict()
            continue
        if m.group(6) is not None and m.group(7) is not None:
            rings[ring_id][m.group(6)] = m.group(7)
            continue

    return rings


ID_MAP = { '0': 'dedicated', '1': 'backup' }

def get_id_name(idnum):
    return ID_MAP.get(idnum, 'RING_ID={n!s}'.format(n=idnum))

def do_check(logger, options):
    rc, faulty = 0, list()

    rings = cfgtool_status(logger)

    if rings is not None:
        for num, data in rings.items():
            status = data.get('status', 'unknown status')
            if status.find('active with no faults') != -1:
                continue
            rc = -1
            faulty.append(get_id_name(num))

        # Create the status msg
        if len(faulty) != 0:
            message = 'FAULTY state in {ids} interface{s}'.format(ids=' and '.join(sorted(faulty)), s=('s' if len(faulty) != 1 else ''))
        else:
            message = 'OK'
    else:
        message = 'Cluster not running'
        rc = 0

    # Create output
    doc = xml.Document()

    root = doc.createElement('program');
    name = doc.createElement('name')
    text = doc.createTextNode("Cluster Rings")
    name.appendChild(text)
    root.appendChild(name)

    mesg = doc.createElement('message')
    text = doc.createTextNode(message)
    mesg.appendChild(text)
    root.appendChild(mesg)

    # output result
    return rc, root


def main():
    parser = OptionParser()
    parser.add_option('-p', '--pretty', action='store_true', default=False, help="pretty print XML output")
    (options, args) = parser.parse_args()

    # Create logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    logger.addHandler(logging.StreamHandler())

    # Execute Check
    rc, xmldata = do_check(logger, options)
    sys.stdout.write(xmldata.toprettyxml() if options.pretty else xmldata.toxml())
    sys.stdout.flush()
    return rc

if __name__ == '__main__':
    sys.exit(main())
