#!/usr/bin/env python
# vim: tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab
"""
* Copyright (C) 2011   Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* William Adam <william.adam@sangoma.com>
*
* This code is Sangoma Technologies Confidential Property.
* Use of and access to this code is covered by a previously executed
* non-disclosure agreement between Sangoma Technologies and the Recipient.
* This code is being supplied for evaluation purposes only and is not to be
* used for any other purpose.
"""

#Include this directory to  python sys.path
import paths

import sys
import os
import subprocess
import re
import xml.dom.minidom as xml
import fs_helper
import simplejson as json

from optparse import OptionParser
import logging
import logging.handlers

# Usage:
#   sng_netiface_status.py --ifname=x
#     Check for network interface x status (carrier and up/down)
# 
# Output format: (This tool is for monit processing)- NO pretty XML print (use
# inline)
"""
<program>  
    <name>Network Interface X</name>
    <message>Output message string </message>
    <status></status>
    <data>
        <count>xx</count>
        <percentage>xx</percentage>
    </data>
</program>
"""
# Upon success, rc=0, data section populated, message set to:
#    OK
#
# Upon limit in percent exceeded, returns -1 and message set to:
#    Active sessions amount of xx% matches resource limit [active sessions>yy%]
#  Where xx is limit specified as argument and yy is current active sessions in %
#
# Upon error, rc=0 (yes I know :) - NO data section - and message set to (fs_cli
# output) example:
#    [ERROR] fs_cli.c:1305 main() Error Connecting [Socket Connection Error]
def do_check(logger, options):
    # Output variables
    rc=-1
    data=None
    error=None
    ifname='eth0'
    carrier=0
    operstate='down'
    status='disconnected'
    flags = 0

    if options.ifname:
        ifname = options.ifname

    # Loopback interface always up :)
    if 'lo' == ifname:
        status = "connected"
        operstate = "up"
        carrier = 1
        rc = 0
    else:
        if os.path.exists("/sys/class/net/" + ifname):
            try:
                # Get status from /sys/class/net
                with open("/sys/class/net/" + ifname + "/carrier") as f:
                    carrier = int(f.read().strip())
            except:
                carrier = None

            with open("/sys/class/net/" + ifname + "/operstate") as f:
                operstate = f.read().strip()

            with open("/sys/class/net/" + ifname + "/flags") as f:
                flags = int(f.read().strip(), 16)

            # Connected and up ?
            if flags & 0x1:
                if carrier and operstate == 'up':
                    status = "connected"
                    rc = 0
            else:
                status = "unconfigured"
                rc = 0
        else:
            status = "not present on system"
            rc = -1

    # Create the status msg
    error = "Interface:{0} {1} [carrier={2}, operstate={3}, flags={4}]".format(
            ifname, status, carrier, operstate, flags)

    # Create output
    doc = xml.Document()
    # inner root
    root = doc.createElement('program');
    s = doc.createElement('name')
    st = doc.createTextNode("Network")
    s.appendChild(st)
    root.appendChild(s)

    # Populate message element
    s = doc.createElement('message')
    st = doc.createTextNode(error)
    s.appendChild(st)
    root.appendChild(s)
    if data:
        root.appendChild(data)

    # output result
    return rc, root


def main():
    #Parse the options
    usage = "usage: %prog [options] arg"
    optParser = OptionParser(usage)
    optParser.add_option('-n', '--name', action='store', type='string',
                        dest='name', metavar='NAME',
                        help="Resource name")

    optParser.add_option('-l', '--ifname', action='store', type='string',
                        dest='ifname', metavar='IFNAME',
                        help="Network interface name")

    optParser.add_option('-p', '--pretty', action='store_true',
                        dest='pretty', metavar='PRETTY',
                        help="Pretty print output")

    (options, args) = optParser.parse_args()

    # Create logger
    logger = logging.getLogger(os.path.basename(os.path.abspath(sys.argv[0])))
    logger.setLevel(logging.INFO)
    handler = logging.StreamHandler()
    logger.addHandler(handler)

    # Execute Check
    rc, root = do_check(logger, options)
    # Check output
    if options.pretty:
        logger.info(root.toprettyxml())
    else:
        logger.info(root.toxml())
    # Report error/success
    sys.exit(rc)


if __name__ == '__main__':
  main()
