<?php
/**
 * FS SIP Profile Class
 *
 * author: Shaunt Libarian
 */

require_once('application/helpers/safe_helper.php');
safe_require_class('object');
safe_require_class('hardware');
safe_module_require_class('fs','application');
safe_module_require_class('sng','transcoding_config');

class Fs_sip_profile_class extends Safe_configurable_object_class
{
    protected $_registration_data_cache = array();
    protected $_registration_list_cache = array();
    protected $_old_display_name = '';
    /**
     * @brief
     *
     * @param[in out] $node
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($node, $parent_name, $name)
    {
      // Parent constructor to invoke unserialize if needed
      parent::__construct($parent_name, $name, $node);
    }

    /**
      * @brief
      *
      * @return
     */
  protected function configure_general()
  {
    // Default interface name
    $def_if = '';
    // Populate IP enumeration with all management interface IP

    $network_module = $this->_node->software()->application()->find_module_by_name('network');
    $if_enum = $network_module->signaling_interfaces_ip();

    $this->add_field('display-name', 'Display Name', 'string', '', 50);
    $this->set_field_rules('display-name','alpha_dash');
    $this->add_field('rtp-timer-name', 'RTP timer name', 'hidden', 'soft', 50);
    // Create object fields
    $this->add_field('user-agent-string', 'User Agent', 'string', '', 50);
    $this->set_field_rules('user-agent-string', 'required');

    $this->add_enum_field('sip-ip', 'SIP IP Address', 'dropdown', $def_if, $if_enum);
    $this->set_field_rules('sip-ip', 'required');
    $this->set_field_help('sip-ip','IP Address being used for SIP signalling.');
    // Add External IP address
    $this->add_field('ext-sip-ip', 'External SIP IP Address', 'string', '', 50);
    $this->set_field_rules('ext-sip-ip','valid_ext_sip_ip');
    $this->set_field_help('ext-sip-ip',
		    array("An IP address string such as '200.1.2.3'",
			"SIP messages in this profile will use the specified IP address in all headers such as Contact, From, etc",
			"<br>The string 'auto-nat'",
			"Gateway will use uPNP or NAT-PMP to discover the public IP address it should use",
			"<br>A string starting with 'stun:' followed by a host name or IP, such as: 'stun:my-stun-server.com' ",
			"Gateway will use the specified STUN server to discover the public IP address",
			"<br>A string starting with 'host:' followed by a host name, such as 'host:my-small-office.com'",
			"Gateway will resolve the given host name to find out the IP address to use (useful for dynamic DNS)")
		    );
    $this->add_field('sip-port', 'Port', 'string', '5060');
    $this->set_field_rules('sip-port','required|valid_port_number|callback_port_check[sip-port]');
    $this->set_field_help('sip-port','Port being used for SIP Signalling');

    $this->add_enum_field('transport', 'Transport', 'dropdown', 'udptcp',
      array(
        'udp'     => 'UDP',
        'tcp'     => 'TCP',
        'tls' => 'TLS',
        'udptcp'  => 'UDP+TCP',
        'udptcptls' => 'UDP+TCP+TLS'
      ) );
    $this->set_field_help('transport','Transport type being used for SIP.');

    $this->add_field('outbound-proxy', 'Outbound Proxy', 'string', '', 50);
    $this->set_field_rules('outbound-proxy', 'valid_ip_port_domain');
    $this->set_field_help('outbound-proxy', 'Optional outbound proxy');

    // RTP IP address
    $rtp_ip_enum = safe_create_enum_list($if_enum, ADD_NONE_ALWAYS, true, '(SIP Profile)');
    $this->add_enum_field('rtp-ip', 'RTP IP address', 'dropdown', '__none__', $rtp_ip_enum);

    $this->set_field_help('rtp-ip',
      array(
        "This is the RTP IP to be used for the RTP network connection.",
        "",
        "If left '(SIP Profile)', the same IP address used for the signaling will be used for RTP.",
        "",
        "If RTP is handled via exposed hardware (ie 'exposed' media mode) this parameter is ignored."
	));

    // External RTP IP address
    $this->add_field('ext-rtp-ip', 'External RTP IP address', 'string', '', 50);
    $this->set_field_rules('ext-rtp-ip','valid_ext_sip_ip');
    $this->set_field_help('ext-rtp-ip',
      array(
	    "An IP address string such as '200.1.2.4'",
	    "The specified IP will be used as the advertised address in the SDP payload",
	    "<br>The string 'auto-nat'",
	    "Gateway will use uPNP or NAT-PMP to discover the public IP address it should use",
	    "<br>A string starting with 'stun:' followed by a host name or IP, such as: 'stun:my-stun-server.com' ",
	    "Gateway will use the specified STUN server to discover the public IP address",
	    "<br>A string starting with 'host:' followed by a host name, such as 'host:my-small-office.com'",
	    "Gateway will resolve the given host name to find out the IP address to use (useful for dynamic DNS)"
      ));

    // Add General category
    $this->set_field_category('display-name','General');
  }

  public function check_session_timeout($field_name)
  {
    $session_timeout = $this->get_data_value('session-timeout');
    $minimum_session_expires = $this->get_data_value('minimum-session-expires');
    if ($session_timeout < $minimum_session_expires) {
      return array('check_session_timeout' => 'Session Expires must be equal or bigger than Minimum Session Expires');
    }
    return true;
  }

  protected function configure_timing()
  {
    $this->add_enum_field('enable-timer', 'SIP Session Timer', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('enable-timer','Enable or Disable the SIP session timer.');

    $this->add_field('session-timeout', 'Session Expires', 'string', 1800);
    $this->set_field_rules('session-timeout','required[enable-timer]|is_numeric|greater_or_equal[90]|callback_check_session_timeout');
    $this->set_field_help('session-timeout','Value in seconds for the Session-Expires header to be used in session expiration requests.');

    $this->add_field('minimum-session-expires', 'Minimum Session Expires', 'string', 120);
    $this->set_field_rules('minimum-session-expires','required[enable-timer]|is_numeric|greater_or_equal[90]');
    $this->set_field_help('minimum-session-expires','Value in seconds for the Min-SE header indicating the minimum acceptable value in seconds for the Session-Expires header.');

    $this->add_field('rtcp-audio-interval-msec', 'RTCP Interval', 'string', '5000', 10);
    $this->set_field_rules('rtcp-audio-interval-msec', 'integer|greater_or_equal[100]|less_or_equal[5000]');
    $this->set_field_help('rtcp-audio-interval-msec','RTCP reports interval in msec.');

    $this->add_field('rtp-timeout-sec', 'RTP Timeout', 'string', 60);
    $this->set_field_rules('rtp-timeout-sec', 'integer|greater_or_equal[0]');
    $this->set_field_help('rtp-timeout-sec','RTP Timeout in seconds. If 0 is specified, timeout is disabled.');

    $this->add_field('timer-T1', 'SIP T1 Timer', 'string', 500);
    $this->set_field_rules('timer-T1', 'integer|greater_than[0]');
    $this->set_field_help('timer-T1','SIP T1 Timer');

    $this->add_field('timer-T1X64', 'SIP T1X64 Timer', 'string', 32000);
    $this->set_field_rules('timer-T1X64', 'integer|greater_than[0]');
    $this->set_field_help('timer-T1X64','SIP T1X64 Timer');

    $this->add_field('timer-T2', 'SIP T2 Timer', 'string', 4000);
    $this->set_field_rules('timer-T2', 'integer|greater_than[0]');
    $this->set_field_help('timer-T2','SIP T2 Timer');

    $this->add_field('timer-T4', 'SIP T4 Timer', 'string', 4000);
    $this->set_field_rules('timer-T4', 'integer|greater_than[0]');
    $this->set_field_help('timer-T4','SIP T4 Timer');

    // Create groups
    $this->create_group('timer', array('session-timeout', 'minimum-session-expires'));
    $this->conditional_control('enable-timer', 'timer');

    // Add Timing category
    $this->set_field_category('enable-timer','Timing');

  }
  protected function configure_interoperability()
  {
    $this->add_enum_field('enable-100rel', '100 Reliability', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('enable-100rel','SIP provisional message reliability.');

    $this->add_enum_field('enable-3pcc', '3PCC', 'dropdown', 'proxy',
		    array('false' => 'Disable', 'proxy' => 'Enable')
		    );
    $this->set_field_help('enable-3pcc','Enable processing of 3PCC (third party call control) to allow processing INVITE messages without an SDP');
    $this->add_enum_field('enable-3pcc-relay-alerting', '3pcc Relay Alerting', 'dropdown', 'false', $this->enable_disable_enum());
    
    $this->add_enum_field('enable-request-dns-caching', 'Request SIP DNS Caching', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('enable-request-dns-caching','Enable SIP DNS caching. Cached value will be used on subsequent outgoing SIP Request with the same URL');
    $this->add_enum_field('ignore-183nosdp', 'Ignore 183 without SDP', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('ignore-183nosdp','Enable or Disable ignoring 183 messages without a specific SDP.');

    $this->add_enum_field('allow-private-wire-info', 'Allow Privite Wire Info', 'dropdown', 'true', $this->enable_disable_enum());
    $this->set_field_help('allow-private-wire-info','Enable or Disable allowing private wire info messages.');

    $this->add_field('fqdn-in-contact', 'FQDN in Contact Header', 'string', '', 50);
    $this->set_field_rules('fqdn-in-contact', 'valid_ip_or_domain');
    $this->set_field_help('fqdn-in-contact','Use FQDN instead of IP address in Contact Header.');

    $this->add_field('max-sip-request-length', 'Maximum SIP Request URI Length', 'string', '', 10);
    $this->set_field_rules('max-sip-request-length', 'integer|greater_than[0]');
    $this->set_field_help('max-sip-request-length','Maximum length of Request URI.');

    $this->add_enum_field('notify-refer-on-final-rsp', 'Notify REFER on Final Response', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('notify-refer-on-final-rsp','Whether to determine REFER result base on final response. This applies only if a new call leg is created to the referee.');

    $this->add_enum_field('rtp-auto-local-port-renew-on-reinvite', 'Renew RTP Port On ReInvite', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('rtp-auto-local-port-renew-on-reinvite',
      array(
        'Enable/Disable Automatic Local RTP Port Renewal On ReInvite.',
        'If enabled, when a new SDP offer is received in connected state and if media line remote IP or Port has been changed, a new local port is assigned for the RTP stream.'
        )
    );

    $this->add_enum_field('allow-asymmetric-dynamic-payload-type', 'Allow Asymmetric Dynamic Payload Type', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('allow-asymmetric-dynamic-payload-type','Allow or Disallow Asymmetric Dynamic Payload Type.');

    $this->add_enum_field('always-add-mode-to-sdp', 'Always add media mode to outgoing SDP', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('always-add-mode-to-sdp',
      array(
        'By default when session level media mode is equal to media line level media mode, the a=<media_mode> line is ommitted ',
        'from the outgoing SDP. Setting this parameter to true will alter that behavior and force to always print the ',
        'm=<media_mode> line to outgoing SDP.')
     );

    $this->add_enum_field('remove-contact-from-route', 'Do not Append Contact to UAS Route', 'dropdown', 'false', $this->enable_disable_enum());
    $this->set_field_help('remove-contact-from-route',
      array(
        'By default sip call with Record-Route headers is received, any outgoing new transaction from the SBC will include the ',
        'received contact header value as last Route which can break interoperability with some proxies. Setting thsi option will ',
        'prevent the SBC from doing that.')
     );
    
	// Add Interoperability category
    $this->set_field_category('enable-100rel','Interoperability');
  }
  protected function configure_encryption()
  {
    //TLS Options
    $this->add_enum_field('TLS/tls-version','TLS Version','dropdown','tlsv1',array(
      'sslv23' => 'SSL Version 2 & 3',
      'tlsv1' => 'TLS Version 1'
    ));
    $this->set_field_help('TLS/tls-version','The version being used by SSL/TLS');

    //Insert Certificate stuff here
    $cert_module = $this->_node->software()->application()->find_module_by_name('certificate');
    $certs = array(
      '__none__' => '(None)'
    );
    foreach($cert_module->get_aggregate_objects('server') as $cert){
      $certs[$cert->name()] = $cert->description();
    }
    $this->add_enum_field('TLS/certificate','TLS Certificate','dropdown','__none__',$certs);
    /*$this->add_upload_field('TLS/certificate','TLS Certificate','upload','',30);
    $this->set_field_help('TLS/certificate','TLS/SSL certificate that will be used with this specific sip profile. Must be a valid PEM file.');
    $this->add_field('TLS/certificate-file-name','TLS Certificate File Name','hidden','',60);
    $this->add_field('TLS/certificate-file-content','TLS Certificate','hidden','',60);*/

    $this->add_field('TLS/tls-passphrase','TLS Passphrase','string','',60);
    $this->set_field_rules('TLS/tls-passphrase','alpha_dash');
    $this->set_field_help('TLS/tls-passphrase','If this SIP Profile is using a private key with a passphrase, you can enter the passphrase here.');

    $this->add_enum_field('TLS/tls-verify-date','Certificate Date Verification','dropdown','true',$this->enable_disable_enum());
    $this->set_field_help('TLS/tls-verify-date','Enable or Disable certificate date verification.');

    $this->add_enum_field('TLS/tls-verify-policy','Certificate Verification Policy','dropdown','out',array(
      'in' => 'Incoming',
      'out' => 'Outgoing',
      'all' => 'Incoming and Outgoing',
      'none' => 'No Verification'
    ));
    $this->set_field_help('TLS/tls-verify-policy','Enable certificate verification policy on incoming, outgoing, or all connections. Can be disabled by selecting No Verification');

    $this->add_field('TLS/tls-sip-port','TLS Port','string',5061,20);
    $this->set_field_rules('TLS/tls-sip-port','valid_port_number|greater_than[1024]|less_than[65525]|callback_port_check[TLS/tls-sip-port]');
    $this->set_field_help('TLS/tls-sip-port','TCP Port required for secure SIP signalling');

    $tls_group = array('TLS/tls-version', 'TLS/certificate', 'TLS/tls-passphrase', 'TLS/tls-verify-date', 'TLS/tls-verify-policy', 'TLS/tls-sip-port');
    $tls_condition = array();
    foreach($tls_group as $item){
        $tls_condition['udp'][] = array('field_name' => $item, 'status' => 'disable');
        $tls_condition['tcp'][] = array('field_name' => $item, 'status' => 'disable');
        $tls_condition['tls'][] = array('field_name' => $item, 'status' => 'enable');
        $tls_condition['udptcp'][] = array('field_name' => $item, 'status' => 'disable');
        $tls_condition['udptcptls'][] = array('field_name' => $item, 'status' => 'enable');
    }
    $this->create_group('tls', $tls_group,
                array('default' => array( 'value' => 'udptcp', 'status' => 'disable'),
                      'condition' =>$tls_condition
                )
            );
    $this->conditional_control('transport', 'tls');


    $this->add_enum_field('TLS/enable-secure-media','Secure RTP','dropdown','false',$this->enable_disable_enum());
    $this->set_field_help('TLS/enable-secure-media','Enable or Disable Secure RTP.');

    $this->add_enum_field('srtp/require-secure-rtp','Require Only Secure RTP','dropdown','false',$this->enable_disable_enum());
    $this->set_field_help('srtp/require-secure-rtp','Require inbound leg INVITE to include secure rtp offer.');

    $this->add_enum_field('srtp/force-outbound-secure-rtp','Force outbound Secure RTP','dropdown','false',$this->enable_disable_enum());
    $this->set_field_help('srtp/force-outbound-secure-rtp','Force outbound leg INVITE to include secure rtp offer irrespective of inbound leg media. <br> Software mode by default does not add SRTP on outbound INVITE always');

    $this->add_enum_field('srtp/support-sdp-secure-avp','Secure AVP','dropdown','false',$this->enable_disable_enum());
    $this->set_field_help('srtp/support-sdp-secure-avp', array(
			    'This option breaks RFC 3711 and is only valid when Lync Interoperability is enabled.',
			    '',
			    'When this option is enabled, SDP media lines will use RTP/AVP even when SRTP is enabled',
			    'and an a=crypto line will be added as usual for SRTP sessions.',
			    '',
			    'The standard RTP/SAVP profile will not be used for SDP secure media lines when this option is enabled.',
			    ''
                         ));
    $secureavp_message = "Please read help tip before enabling this setting, Enable if only required";
    $secureavp_change = "if (!this.selectedIndex) warnOnInput(event,'" . strval($secureavp_message) . "');";
    $this->set_field_onchange('srtp/support-sdp-secure-avp', $secureavp_change);

    $this->add_enum_field('srtp/crypto-optional-lifetime','Crypto Life Time','dropdown','__disable__',array(
      '__disable__' => 'Disable',
      '2^48' => 'High',
      '2^31' => 'Medium',
      '2^16' => 'Low',
    ));
    $this->set_field_help('srtp/crypto-optional-lifetime','Number of packets the SRTP crypto master key is valid for (RFC4568 lifetime optional sdp parameter). ');

    $this->add_enum_field('srtp/crypto-optional-mki-length-string','Crypto MKI Length','dropdown','__disable__',array(
      '__disable__' => 'Disable',
      '1:1' => '1:1',
    ));
    $this->set_field_help('srtp/crypto-optional-mki-length-string','The MKI length is the size of the MKI field in the SRTP packet.');

    // SRTP group
    $this->create_group('srtp', array('srtp/require-secure-rtp', 'srtp/force-outbound-secure-rtp', 'srtp/support-sdp-secure-avp', 'srtp/crypto-optional-lifetime', 'srtp/crypto-optional-mki-length-string'));
    $this->conditional_control('TLS/enable-secure-media', 'srtp');

    // Add Encryption  category
    $this->set_field_category('TLS/tls-version','Encryption');
  }

  public function configure(){
    // General configuration
    $this->configure_general();
    // General Interop
    $this->configure_interoperability();
    // General Timer
    $this->configure_timing();
    // Encryption configuration
    $this->configure_encryption();
    $this->register_aggregate_object('registration',
            array(
                    'name' => 'Profile Registration',
                    'configurable' => false,
                    'pagination' => true,
                    'dynamic' => true,
                    'base_path' => false,
                    'autoname' => true,
                    'global_methods' => true,
                    'methods' => array(
                            'retrieve' => array(
                                    'name' => 'Retrieve',
                                    'description' => 'Retrieve a registration',
                                    'request' => 'GET',
                                    'scope' => 'both'
                            ),
                            'flush' => array(
                                    'name' => 'Flush',
                                    'description' => 'Delete a registration',
                                    'request' => 'POST',
                                    'scope' => 'both'
                            ),
                    ),
            )
    );

    return parent::configure();
  }
    public function prepare_edit()
    {
        parent::prepare_edit();
        // Check if system is running and object as been serialized
        $the_app = $this->_node->software()->application();
        if (Safe_service_class::STATUS_RUNNING == $the_app->status()
          && Safe_object_serializer_class::OBJ_STATUS_NEW != $this->status()) {
            // Check profile status
            $sip_module = $the_app->find_module_by_name('sip');
            if($profile_status = $sip_module->api_status_profile($this->display_name())){
              $ro_fields = array(
                'display-name',
                'sip-ip',
                'ext-sip-ip',
                'transport',
                'sip-port',
                'rtp-ip',
                'enable-request-dns-caching'
                );
              // Prevent binding edition
              foreach($ro_fields as $f){
                $this->set_field_disabled($f, true);
                // Adjust help so user knows why :)
                $help = $this->get_field_help($f);
                if(!is_array($help)){
                  $help = array($help);
                }
                $help = array_merge(
                array(
                  '<font color="red">This parameter cannot be changed because Profile is started.</font>',
                  '',
                ), $help);
                $this->set_field_help($f, $help);

              }
            }
        }

    }

  // Application helpers
  public function get_sip_transport_info(&$transport, &$port,&$ip_addr=null){
    $transport = $this->get_data_value('transport', false);
    $port = $this->get_data_value('sip-port');
    $ip_addr = $this->get_data_value('sip-ip',false);
  }
  public function get_ip_obj($ip_name)
  {
      $tmp_ip_obj = null;
      $network_module = $this->node()->software()->application()->find_module_by_name('network');
      $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
      $failover_mode = ($cluster_module != NULL && $cluster_module->get_cluster_mode() !== 'disabled');
      if ($failover_mode) {
          if (strpos($ip_name, 'floating/') === 0) {
              $ip_name = substr($ip_name, 9);
              $tmp_ip_objs = $cluster_module->floating_ips();
              foreach ($tmp_ip_objs as $ip_obj) {
                  if ($ip_obj->name() == $ip_name) {
                      $tmp_ip_obj = $ip_obj;
                      break;
                  }
              }
          }else if (strpos($ip_name, 'network/') === 0) {
              //loop on lo interface find the ip obj
              $ip_address = substr($ip_name, 8);
              $loopback_ips = $network_module->get_ips('lo');
              if($loopback_ips){
                  foreach ($loopback_ips as $ip_obj){
                      if ($ip_obj->address() == $ip_address) {
                          $tmp_ip_obj = $ip_obj;
                          break;
                      }
                  }
              }
          }
      } else {
          $tmp_ip_obj = $network_module->get_ip_by_name($ip_name);
      }
      return $tmp_ip_obj;
  }
  public function get_ip_address($ip_name)
  {
      $tmp_ip = '';
      $tmp_ip_obj = $this->get_ip_obj($ip_name);
      if($tmp_ip_obj){
          $tmp_ip = $tmp_ip_obj->address();
      }
      return $tmp_ip;
  }
  public function port_check($field_name = null, $ip_address = '', $port_value = null){
      if($field_name){
          $port_name = $this->get_data_label($field_name);
          $port_value = $this->get_data_value($field_name);
          $port_type  = $this->get_data_value('transport', false);
          if($field_name == 'TLS/tls-sip-port'){
              if(strpos(strtoupper($port_type),'TLS') === false){
                  return true;
              }else{
                  //check if SIP tcp port = SIP tls port
                  if($port_value == $this->get_data_value('sip-port') &&  strpos(strtoupper($port_type), 'TCP') !== false){
                      return array('port_check' => 'The TLS Port field matches the Tcp SIP port field.');
                  }else{
                      $port_type = 'TCP';
                  }
              }
          }
      }else{
          return true;
      }
      //array for if -> ip
      $module = $this->node()->software()->application()->find_module_by_name('network');

      if (!is_numeric(preg_replace('#\.#','',$ip_address)) && !preg_match('#^[0-9a-z]{2}:([0-9a-z]{2}:){4}[0-9a-z]{2}$#i',$ip_address)) {
        $ip_obj = $this->get_ip_obj($this->get_data_value('sip-ip',false));
        if($ip_obj){
            $ip_address = $ip_obj->address();
        }
      }

      //ip port
      $ports = array();
      if($ip_address){
          $ports[] = array('name' =>$port_name,'port'=>$port_value, 'ip'=>$ip_address, 'type' => $port_type);
      }

      //other sip profiles
      $sip_module = $this->node()->software()->application()->find_module_by_name('sip');
      $profiles = $sip_module->profiles();
      $error_msg ='';
      foreach ($profiles as $profile) {
          $ip_port_resource = array();
          if ($profile->name() != $this->name()) {
              $profile->get_sip_transport_info($transport, $port, $interface);
              $tmp_ip_obj = $profile->get_ip_obj($interface);
              $tmp_ip = '';
              if($tmp_ip_obj){
                  $tmp_ip = $tmp_ip_obj->address();
              }
              $ip_port_resource[] = array('name' =>$profile->display_name(),'port'=>$port, 'ip'=> $tmp_ip, 'type' => $transport);
              if(strpos(strtoupper($transport),'TLS') !== false){
                  $ip_port_resource[] = array('name' =>$profile->get_data_label('TLS/tls-sip-port'),'port'=>$profile->get_data_value('TLS/tls-sip-port'), 'ip'=>$tmp_ip, 'type' => 'TCP');
              }
              foreach($ports as $port_item){
                  foreach($ip_port_resource as $resource_item){
                      if(safe_check_port_conflict($port_item, $resource_item)){
                          $error_msg = $error_msg . " with {$profile->display_name()} at port {$resource_item['port']}, " ;
                      }
                  }
              }
          }
      }
      //check $conflicts
      $conflicts = $this->node()->software()->application()->check_resource_conflict('sip','IP_PORT', $ports);
      foreach($conflicts as $conflict){
          $mod = $this->node()->software()->application()->find_module_by_name($conflict['name']);
          $name = $mod->description();
          $error_msg = $error_msg . " with {$name} at {$conflict['desc']}, " ;
      }

      $error_msg_2 ='';
      //check system port
      $app_process_name = $this->node()->software()->application()->process_name();
      $user_prog = "";

      foreach($ports as $port_item){
          if(!$this->node()->os()->valid_ip_port($port_item['ip'], $port_item['port'], $port_item['type'], $app_process_name, $user_prog)){
            // Are we running our process under known dbg process ?
            $app_dbg_processes = array('valgrind');
            if(!in_array($user_prog, $app_dbg_processes)){
              $error_msg_2 = " This port is already being used by another program. " ;
            }
          }
      }
      if($error_msg == '' && $error_msg_2 == ''){
          return true;
      }else{
          if ($error_msg!=''){
              $error['port_check'] = 'This port has conflict'.substr($error_msg, 0, -2).'. ';
          }
          if ($error_msg_2!=''){
              $error['port_check'] = $error['port_check'] . $error_msg_2 .' ';
          }
          return $error;
      }
  }
  /**
   * @brief Check transport compatible with SIP profile transport
   *
   * @param[in out] $transport : transport string (udp, tcp...)
   *
   * @return
   */
  public function check_transport($transport)
  {
    $_transport = $this->get_data_value('transport', false);
    return (FALSE === strpos($_transport, $transport))?false:true;
  }
  public function get_certificate_info(&$cert_name)
  {
    if($this->check_transport('tls')){
        $cert_name = $this->get_data_value('TLS/certificate', false);
        if ($cert_name == '__none__'){
            return false;
        }else{
            return true;
        }
    }
    return false;
  }
  public function summary($type = 'horizontal' , $long = false){
      $desc_custom = array('user-agent-string', 'call-routing', 'sip-ip', 'sip-port','transport');
      $table_line = array("Profile Name" => $this->display_name());
      foreach ($desc_custom as $desc_key){
          $table_line[$this->get_data_label($desc_key)] = $this->get_data_value($desc_key);
      }
      $new_field = 'auth-calls-by-network-address';
      if($this->get_data_value($new_field,false) != 'false'){
          $table_line[$this->get_data_label($new_field)] = array(
                  'data' => '<font color="red">' .$this->get_data_value($new_field) .'</font>',
                  'formatter' => 'none',
                  );
      }else{
          $table_line[$this->get_data_label($new_field)] = $this->get_data_value($new_field);
      }
      return array('data' => $table_line);
    }

    public function api_retrieve_registration($name, $data = null, &$output = null){
        $path = $this->object_name();
        $module = $this->node()->software()->application()->find_module_by_path($path);
        if($name){
            //return one record with uniqe call-id
            $call_id = safe_set_default($data, 'call_id', urldecode($name));
            if(!isset($this->_registration_data_cache[$call_id])){
                $reg_output = $module->api_registration_profile($this->display_name(), array('select' => '', 'call_id' => $call_id), $output);
                if($reg_output === false){// no record found
                    $output['message'] = 'Not Found';
                    return false;
                }else{
                    //save to registration_data_cache, key using  urlencode(call_id)
                    $this->_registration_data_cache[$call_id] = $reg_output[0];
                }
            }
            return $this->_registration_data_cache[$call_id];
        }else{
            //return list
            $cache_key = md5(implode('', $data));
            if(!isset($this->_registration_list_cache[$cache_key])){
                $params_total = array( 'select' => 'count(*) as total', 'limit' => 1, 'offset' => 0);
                $rs_total = $module->api_registration_profile($this->display_name(), $params_total, $output);
                $this->_registration_list_cache[$cache_key]['total'] = $rs_total[0]->total;
                $limit = safe_set_default($data,'pagination/limit',100);//max return record number
                $offset = safe_set_default($data,'pagination/offset',0);//start from record number
                $params = array( 'limit' => $limit, 'offset' => $offset, 'select' => '' );
                $this->_registration_list_cache[$cache_key]['param'] = $params;
                $this->_registration_list_cache[$cache_key]['data'] = array();
                if($rs_total[0]->total > 0){
                    $reg_output = $module->api_registration_profile($this->display_name(), $params, $output);
                    foreach($reg_output as $reg_item){
                        $this->_registration_data_cache[urlencode($reg_item['call-id'])] = $reg_item;
                        $this->_registration_list_cache[$cache_key]['data'][urlencode($reg_item['call-id'])] =  $reg_item;
                    }
                }
            }
            $output['total'] = $this->_registration_list_cache[$cache_key]['total'];
            $output['limit'] = $this->_registration_list_cache[$cache_key]['param']['limit'];
            $output['offset'] = $this->_registration_list_cache[$cache_key]['param']['offset'];
            $output['data'] = $this->_registration_list_cache[$cache_key]['data'];
            return $output;
        }
    }
    public function api_flush_registration($name = '', $data = null, &$output = null){
        $path = $this->object_name();
        $module = $this->node()->software()->application()->find_module_by_path($path);
        $call_id = '';
        if ($name != '') $call_id = urldecode($name);
        $params = array( 'call_id' => $call_id );
        return $module->api_flush_registration_profile($this->name(), $params, $output);
    }
    public function validate($data, &$output){
      if (!$this->remap_ip_objects_validate($data)) {
        return false;
      }
      $reserved = ['common'];
      if (in_array($this->display_name(), $reserved)) {
        $this->set_error('display-name', $this->display_name() . ' is a reserved and cannot be used as display name');
        return false;
      }
      $this->_old_display_name = $this->display_name();
        if(isset($data['transport'])){
            $tmp_data = $data['transport'];
        }else{
            $tmp_data = $this->get_data_value('transport', false);
        }
        if(false === strpos($tmp_data, 'tls')){
            $data['TLS/certificate'] = '__none__';
        }
        $res = parent::validate($data, $output);
        if ($this->node()->software()->application()->check_display_name_conflict('sip', 'profile', $this->display_name(), $this->name())) {
          $this->set_error('display-name', $this->display_name() .' already exists.', $output);
          return false;
        }
        return $res;
    }
    /**
     * @brief Save method - Override to send obj modified event
     *
     * @return
     */
    public function save()
    {
        $result = parent::save();
        if($result){
            //send sip profile modification event out
            //to trigger firewall add/remove rules
            $path_info = $this->path_info();
            $receivers = $this->node()->software()->application()->check_object_modified($this, $path_info['module'], $path_info['type'], $this->name());
        }
        return $result;
    }
    public function dispose()
    {
        if(parent::dispose()){
            $path_info = $this->path_info();
            $receivers = $this->node()->software()->application()->check_object_modified($this, $path_info['module'], $path_info['type'], $this->name());
            return true;
        }
        return false;
    }
    /**
     * @brief get_port method - return the port used in this profile
     *
     * @return
     */
    public function get_port()
    {
        $port = array();
        $transport =  $this->get_data_value('transport', false);
        $path_info = $this->path_info();
        $mod = $this->node()->software()->application()->find_module_by_name($path_info['module']);
        $desc = $mod->description() . ' ' . $mod->aggregate_object_name($path_info['type'])  . ' ' . $this->name();
        if($transport != 'tls'){
            $port[] = array(
                        'protocol' => $transport,
                        'field'=> 'sip-port',
                        'port_begin' => $this->get_data_value('sip-port', false),
                        'port_end' => $this->get_data_value('sip-port', false),
                        'desc' => $desc . ' ' . $this->get_data_label('sip-port')
                    );
        }
        if($this->check_transport('tls')){
            $port[] = array(
                        'protocol' => 'tls',
                        'field'=> 'TLS/tls-sip-port',
                        'port_begin' => $this->get_data_value('TLS/tls-sip-port', false),
                        'port_end' => $this->get_data_value('TLS/tls-sip-port', false),
                        'desc' => $desc . ' ' . $this->get_data_label('TLS/tls-sip-port'));
        }
        return $port;
    }
    public function get_rtp()
    {
        $sip_ip =  $this->get_data_value('sip-ip', false);
        $rtp_ip =  $this->get_data_value('rtp-ip', false);

        $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
        $transcoding_mode = $sngms_module->config();
        $mode = $transcoding_mode->ip_mode();
        //Check if the rtp-ip field is set in the SIP profile. If it is, set $profile_rtp_ip to that value,
        //if it's not, then use the sip-ip to set $profile_rtp_ip
        if($rtp_ip == '__none__' || $rtp_ip == ''){
            $rtp_ip = $sip_ip;
        }
        return $rtp_ip;
    }

  /* remap addresses to/from HA mode */
  public function remap_ip_object_field(&$data, $field_name, $field_data, $network_module, $cluster_module, $cluster_enable)
  {
    if ($field_data == '__none__')
        return true;

    $strdata = explode('/', $field_data);

    if ($cluster_enable) {
        // non-HA => HA
        if (count($strdata) == 1) {
            $ipname = $strdata[0];
            $ipdata = $network_module->get_ip_by_name($ipname);
            if ($ipdata == null) {
                safe_log(SAFE_LOG_INFO, 'could not find IP object "' . $field_data . '" for remapping address to HA mode');
                return true;
            }

            $ifname = $ipdata->get_interface_name();
            if (strcmp($ifname, "lo") == 0) {
                $data[$field_name] = 'network/' . $ipdata->address();
                return true;
            }

            $ipcidr = safe_cidr2network($ipdata->address(), $ipdata->prefix());
            $ipmatches = array();
            foreach ($cluster_module->get_floating_ips($ifname) as $fipobj) {
                $fipcidr = safe_cidr2network($fipobj->address(), $fipobj->prefix());
                if (strcmp($ipcidr, $fipcidr) != 0)
                    continue;

                if ($this->port_check('sip-port', $fipobj->address()) !== true)
                    continue;

                $ipmatches[] = $fipobj;
            }

            if (count($ipmatches) == 1) {
                safe_log(SAFE_LOG_INFO, 'mapping IP "' . $ipdata->address() . '" to floating IP "' . $fipobj->address() . '"');
                $data[$field_name] = 'floating/' . $fipobj->name();
                return true;
            }

            if (count($ipmatches) > 1) {
                $strmatches = array();
                foreach($ipmatches as $ipmatch) { $strmatches[] = $ipmatch->address(); }
                safe_log(SAFE_LOG_INFO, 'multiple floating IPs on ' . $ifname . '/' . $ipcidr .
                    ' (' . implode(', ', $strmatches) . '): not remapping "' .
                        $ipdata->address() . '" to HA mode');
                return true;
            }

            safe_log(SAFE_LOG_INFO, 'could not find floating IP on ' . $ifname . '/' . $ipcidr .
                 ': not remapping "' . $ipdata->address() . '" to HA mode');
            return true;
        }

    } else if ($cluster_module) {
        // HA => non-HA
        if (count($strdata) == 2) {
            if (strcmp($strdata[0], "network") == 0) {
                foreach ($network_module->get_aggregate_objects('ip') as $ipname => $ipdata) {
                    if (strcmp($ipdata->address(), $strdata[1]) != 0)
                        continue;

                    $data[$field_name] = $ipdata->name();
                    return true;
                }
                safe_log(SAFE_LOG_INFO, 'could not find IP object "' . $field_data . '" - not remapping to non-HA mode');
                return true;

            } else if (strcmp($strdata[0], "floating") == 0) {
                $fipobj = $cluster_module->floating_ips($strdata[1]);
                if ($fipobj == null) {
                    safe_log(SAFE_LOG_INFO, 'could not find floating IP object "' . $strdata[1] . '" - not remapping to non-HA mode');
                    return true;
                }

                $fipcidr = safe_cidr2network($fipobj->address(), $fipobj->prefix());
                $ipmatches = array();
                foreach ($network_module->get_ips($fipobj->get_interface_name()) as $ipobj) {
                    $ipcidr = safe_cidr2network($ipobj->address(), $ipobj->prefix());
                    if (strcmp($fipcidr, $ipcidr) != 0)
                        continue;

                    if ($this->port_check('sip-port', $ipobj->address()) !== true)
                        continue;

                    $ipmatches[] = $ipobj;
                }

                if (count($ipmatches) == 1) {
                    safe_log(SAFE_LOG_INFO, 'mapping floating IP "' . $fipobj->address() . '" to IP "' . $ipobj->address() . '"');
                    $data[$field_name] = $ipobj->name();
                    return true;
                }

                if (count($ipmatches) > 1) {
                    $strmatches = array();
                    foreach($ipmatches as $ipmatch) { $strmatches[] = $ipmatch->address(); }
                    safe_log(SAFE_LOG_INFO, 'multiple IPs on ' . $fipobj->get_interface_name() .
                        '/' . $fipcidr . ' (' . implode('", "', $strmatches) . '): not remapping "' .
                            $fipobj->address() . '" to non-HA mode');
                    return true;
                }

                safe_log(SAFE_LOG_INFO, 'could not find IP on ' . $fipobj->get_interface_name() .
                        '/' . $fipcidr . ': not remapping "' . $fipobj->address() . '" to non-HA mode');
                return true;
            }

            safe_log(SAFE_LOG_INFO, 'unknown IP option "' . $field_data . '" - unable to remap to non-HA mode');
            return true;
        }
    }

    return true;
  }

  public function remap_ip_objects_validate(&$data)
  {
    $network_module = $this->node()->software()->application()->find_module_by_name('network');

    $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
    $cluster_enable = $cluster_module !== NULL && $cluster_module->get_cluster_mode() != 'disabled';

    foreach (array('sip-ip', 'rtp-ip') as $field_name) {
        $field_data = $data[$field_name];
        if ($field_data == null) {
            continue;
        }
        if (!$this->remap_ip_object_field($data, $field_name, $field_data, $network_module, $cluster_module, $cluster_enable)) {
            return false;
        }
    }

    $this->set_data_values($data, false);
    return true;
  }

  public function remap_ip_objects(&$data="")
  {
    $data = $this->get_data_values(false);
    if (!$this->remap_ip_objects_validate($data)) {
        return false;
    }
    $this->set_data_values($data, false);
    $this->save();
    return true;
  }


}

?>
