<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Data table class
 * @author Santiago Tramoyeres Cuesta <scuesta@sangoma.com>
 *         
 */

  /**
   * This class is an abstraction layer for easy implementation of server-side
   * dataTable.js. It uses a kind of Data Source instance (File data source, SQL
   * data source, ...) and performs filtering and pagination· It implements the
   * Iterator interface to let it be used in foreach loop. For example:
   * 
   * Example1: $data_table = new Safe_data_table(array('data_source'=>new Safe_file_data_source_class('/home/user',true),
   *                                                         'title'=>'Table - title',
   *                                                         'rows_per_page'=>20));
   *           foreach($data_table->find(array('date'=>"between '2016-01-01' and '2016-01-31'")) as $rows)
   *              // here rows is an instance of data source
   *              foreach($rows as $pos => $row)
   *                    ...
   * 
   * For iterate over page results Safe_data_table_class implements each_row
   * method that let us execute a callback for each rows for a given page. The
   * callback will get the row position in page, the row itself and the
   * arguments we want to have inside callback function.
   *
   * Example2: $data_table = new Safe_data_table(array('data_source'=>new Safe_file_data_source_class('/home/user',true),
   *                                                         'title'=>'Table - title',
   *                                                         'rows_per_page'=>20));
   *           $args = (object) array('return'=>array());
   *           $data_table->each_row(0,function($i,$row,&$args){
   *                                  print "Position: ".$i." file ".$row['file']." will be returned\n";
   *                                  $args->return[] = $row['file'];
   *                                 },$args);
   *           print_r($args->return);
   *
   * The filter predicate struct depends on Data Source instance
   * used. Safe_file_data_source uses an array, but other data source could use string
   *           
   */
class Safe_data_table_class implements Iterator{
    private $_ds;
    private $_cols;
    private $_title;
    private $_rows_per_page;
    private $_cur_page;
    private $_query;
    private $_settings;

    public static $default_settings = array('bJQueryUI'=>true,
                                            'sPaginationType'=>"four_button",
                                            'sDom'=>'<"top"<"css_left"f><"css_left"l><"css_left"i><"css_right"p>><"css_clear"t><"bottom"><"clear">',
                                            'oLanguage'=>array('sSearch'=>'<i class="fa fa-search fa-fw safe_info" style="margin-top: 6px;margin-left: 2px;float: left;"></i>',
                                                               'sLengthMenu'=>"_MENU_"),
                                            'aaSorting'=>array(),
                                            );
    /**
     * @brief Constructor method
     * @param array  Array with params most important of them should be data_source instance
     */
    public function __construct($args=array()){
        $this->_ds = isset($args['data_source'])? $args['data_source'] : null;
        $this->_cols = isset($args['cols'])? $args['cols'] : array();
        $this->_title = isset($args['title'])? $args['title'] : 'No title';
        $this->_rows_per_page = isset($args['rows_per_page'])? $args['rows_per_page'] : 20;
        $this->_cur_page = 0;
        $this->_query = isset($args['query'])? $args['query'] : array();
        $this->_settings = isset($args['settings'])? self::extend(self::$default_settings,$args['settings']) : self::$default_settings;
        $this->rows_per_page($this->_rows_per_page);
    }

    /**
     * @brief Merges several arrays
     * @param array,array,array
     * return array
     */
    private static function extend(){
        $numArgs = func_num_args();
        $dest = $numArgs>0? func_get_arg(0) : array();
        if($numArgs>1)
            for($i=1;$i<$numArgs;$i++){
                $aux = func_get_arg($i);
                $dest = array_merge($dest,$aux);
            }
        return $dest;
    }

    /**
     * @brief Getter/Setter for dataTable settings
     * @param array   Array with dataTable settings
     * @return array
     */
    public function settings($settings=null){
        if($settings==null)
            return $this->_settings;
        else if(is_array($settings))
            $this->_settings = $settings;
    }

    /**
     * @brief Getter/Setter for number of rows per page. When it is set, the
     * dataTable parameter for this matter is also modified
     * @param integer   Number of rows per page
     * @return integer
     */
    public function rows_per_page($rows_per_page=null){
        if($rows_per_page==null)
            return $this->_rows_per_page;
        $this->_settings['iDisplayLength'] = $rows_per_page;
        $this->_rows_per_page = $rows_per_page;         
    }

    /**
     * @brief Getter/Setter for Table title
     * @param string
     * @return string
     */
    public function title($title=null){
        if($title==null)
            return $this->_title;
        $this->_title = $title;
    }
    
    /**
     * @brief Getter/Setter for column titles
     * @param array   Array with column titles
     * @return array
     */
    public function cols($cols=array()){
        if(empty($cols))
            return $this->cols;
        $this->_cols = $cols;
    }

    /**
     * @brief Getter/Setter page for current state
     * @param integer  the number of page
     * @return Data Source instance
     */
    public function page($cur_page=null){
        if($cur_page===null)
            return $this->_cur_page;
        $this->_cur_page = intval($cur_page);
        return $this->current();
    }
    
    /**
     * @brief Setter for filter predicate. It save the predicate to be use in
     *        following calls of current method.
     * @param array   filter predicate
     */
    public function find($query=array()){
        $this->_query = $query;
        return $this;
    }

    /**
     * @brief Set the page pointer to the starting point
     */
    public function rewind(){
        $this->_cur_page=0;
    }
    
    /**
     * @brief Uses data source to preform filtering task
     * @return Data Source instance
     */
    public function current(){
        return $this->_ds->offset($this->_cur_page*$this->_rows_per_page)->limit($this->_rows_per_page)->find($this->_query);
    }
    
    /**
     * @brief Increase the number of page
     */ 
    public function next(){
        $this->_cur_page++;
    }

    /**
     * @brief Returns the number of current page
     * @return integer
     */
    public function key(){
        return $this->_cur_page;
    }

    /**
     * @brief Check if current page is valid to be accessed   
     * @return bool
     */
    public function valid(){
        return ($this->_cur_page*$this->_rows_per_page)<$this->_ds->length();
    }
    
    /**
     * @brief Returns the number rows. If filter predicate is NOT empty, it
     *        returns the number of total rows that matches the predicate. If filter
     *        predicate is empty, it returns the number of total rows.
     * @return integer
     */
    public function total($query=array()){
        return $this->_ds->length($query);
    }

    /**
     * @brief Returns the total number of pages
     * @return integer
     */
    public function count(){
        return intval($this->_ds->length()/$this->_rows_per_page);
    }

    /**
     * @brief This method iterates and executes a callback over each row of the page.   
     * @param integer  Page number to get access
     * @param function Callback to be execute for every row
     * @param array    Arguments to pass on callback
     */
    public function each_row($page,$callback,&$args=array()){
        $rows = $this->page($page);
        $i=0;
        while(($row =$rows->current()) && ($callback($i++,$row,$args)!==FALSE)){
            $rows->next();
        }
    }
}

?>