<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2017  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * DB data source
 * @author Santiago Tramoyeres Cuesta <scuesta@sangoma.com>
 *         
 */
require_once(dirname(__FILE__).'/safe_data_source_interface.php');
require_once(dirname(__FILE__).'/safe_object_class.php');

class Safe_db_data_source_class implements Safe_data_source_interface, Iterator{

    private $_results;
    private $_class;
    private $_node;
    private $_limit;
    private $_offset;
    private $_count;
    private $_as_object;
    private $_data_filter;
    private $_page_length;
    
    const SAFE_TABLE = 'safe_object';
    /**
     * @brief Constructor method
     * @param string Class name of the return value on each iteration.
     * @param Object The node required to create objects from Class.
     */
    public function __construct($class,&$node,$as_object=true){
        $this->_class = $class;
        $this->_result = array();
        $this->_node = $node;
        $this->_limit = 10;
        $this->_offset = 0;
        $this->_count = 0;
        $this->_as_object = $as_object;
        $this->_data_filter = array();
        $this->_results = array();
        $this->_page_length = 0;
    }

    /**
     * @brief Fields returned for each row iteration
     * @return array An array with all configuration fields from Class
     */
    public function fields(){
        $class = $this->_class;
        $obj = $class::model($this->_node);
        return method_exists($obj,'table_column_titles')? $obj->table_column_titles() : array_keys($obj->get_data_labels());
    }

    private function _clean_results(){
        $length=count($this->_results);
        while(--$length>=0){
            unset($this->_results[$length]);
        }
        $this->_results=array();
        $this->_results_lenght=0;
    }
    
    private function free_result(){
        $this->_count = 0;
        $this->_row = null;
        // Todo get the class_name to ensure we have the right object
        if(is_object($this->_result)){
            $this->_result->free_result();
            unset($this->_result);
        }
        $this->_clean_results();
        $this->_data_filter = array();
        $this->_result = array();
        return $this;
    }


    private function _prepare($query=array()){
        if(!is_array($query))
            return $this->_class? array('class'=>$this->_class) : array();
        foreach($query as $qkey => $qvalue){
            if($qkey=='meta' && is_array($qvalue)){
                foreach($qvalue as $metakey => $metavalue){
                    $query[$metakey] = $metavalue;
                }
                unset($query['meta']);
            }else if($qkey=='like' && is_array($qvalue)){
                continue;
            }else{
                if(!isset($query['data']))
                    $query['data'] = array();                
                $query['data'][$qkey] = $qvalue;
                unset($query[$qkey]);
            }
        }
        return $query;
    }

    /**
     * @brief Filtering method
     * @param array filter predicate
     * @return Safe_db_data_source_class current object
     */
    public function find($query=array()){
        $this->free_result();
        $db = Safe_object_serializer_class::get_serializer()->get_db();
        $query = $this->_prepare($query);
        if($this->_class)
            $query['class']=$this->_class;
        if(isset($query['data']) && is_array($query['data'])){
            $this->_data_filter = $query['data'];
            unset($query['data']);
        }
        if(isset($query['like']) && is_array($query['like'])){
            $db->like($query['like']);
            unset($query['like']);            
        }
        $this->_result = $db->get_where(self::SAFE_TABLE,$query);
        $this->_go_to_offset();
        $this->_fill_results();
        return $this;
    }

    /**
     * @brief Getter/Setter for offset
     * @param integer 
     * @return Safe_file_data_source current object
     */
    public function offset($offset=FALSE){
        if($offest===FALSE)
            return $this->_offset;
        $this->_offset = $offset;
        return $this;
    }

    /**
     * @brief Getter/Setter for limit
     * @param integer
     * @return Safe_file_data_source current object
     */
    public function limit($limit=FALSE){
        if($limit===FALSE)
            return $this->_limit;
        $this->_limit = $limit;
        return $this;
    }

    /**
     * @brief Get the current matched row. This method is requird by Iterator
     * interface and let use Safe_row_data_source in foreach statement.
     * @return object The object from class name. That class should
     * implement the factory method model().
     */
    public function current(){
        $row = ($this->_count<=$this->_page_length)? $this->_results[$this->_count] : null;//->unbuffered_row();
        if($this->_class && $row){
            $data = json_decode($row['data'],true);
            $class = $this->_class;
            $obj = $class::model($this->_node);
            $obj->set_data_values($data);
            $obj->set_name($row['name']);
            $row = $obj;
        }
        return $row;
    }

    /**
     * @brief We are always have to take into account those records that matches the
     * query so the offset will be about those records matching the filter.
     *
     */
    private function _go_to_offset(){
        $offset = $this->_offset;
        while($offset>0 && ($row=$this->_result->unbuffered_row())){
            if($this->_matches($row,$this->_data_filter))
                $offset--;
        }
    }
    
    private function _fill_results(){
        $this->_clean_results();
        $this->_page_length = 0;
        while(($row=$this->_result->unbuffered_row()) && $this->_page_length<$this->_limit){
            if($this->_matches($row,$this->_data_filter)){
                $this->_results[] = $row;
                $this->_page_length++;
            }
        }
    }
    /**
     * @brief As unbuffered_row is moving the pointer to the next row
     * after get the current one, it is not required to increse it
     * from here.
     */
    public function next(){
        $this->_count++;
    }

    /**
     * @brief Returns the number of matched rows for the current page
     * @return integer
     */    
    public function count(){
        return $this->_page_length;
    }

    public function _matches(&$row,$filter){
        $data = json_decode($row['data'],true);
        if($data){
            if($filter!=array()){
                $cmatch = 0;
                foreach($filter as $key => $value){
                    if(isset($data[$key]) && $data[$key]==$value){
                        $cmatch++;
                    }else if(isset($data[$key]) && $data[$key]!=$value){
                        break;
                    }else if(!isset($data[$key])){
                        // If the key is not set in the data lets try with regular
                        // expressions over the key. This is worse than unefficient,
                        // but is the only way we to deal with call routing rules
                        // which have composite attribute keys
                        foreach($data as $dk => $dv){
                            if(preg_match('#'.$key.'#i',$dk) && $value==$dv){
                                $cmatch++;
                                break;
                            }
                        }
                    }
                }
                
                return $cmatch==count($filter);
            }else
                return true;
        }
        return false;
    }
    
    /**
     * @brief If query param is NOT empty, returns the number of matched rows.
     *        If query param is empty, returns the number of rows of
     *        the whole table
     * @param array   filter predicate
     * @return integer
     */
    public function length($filter=array()){
        $db = Safe_object_serializer_class::get_serializer()->get_db();
        $length = 0;
        
        // if(!isset($filter['meta']) && $this->_class)
        //     $filter['meta']=array('class'=>$this->_class);
        // else if($this->_class)
        //     $filter['meta']['class'] = $this->_class;
        $data_query = array();
        $filter = $this->_prepare($filter);
        if($this->_class)
            $filter['class']=$this->_class;
        if(isset($filter['data']) && is_array($filter['data'])){
            $data_query = $filter['data'];
            unset($filter['data']);
        }
        $query = $db->get_where(self::SAFE_TABLE,$filter);

        if($query){
            // If there is something else means that there is a filter for
            // data configuration
            if(count($filter)>0){
                while(($row = $query->unbuffered_row())){
                    if($this->_matches($row,$data_query))
                        $length++;
                }
            }else{
                $length = $query->num_rows();
                $query->free_result();
                unset($query);
            }
        }
        return $length;
    }

    /**
     * @brief Sets the pointer to starting point and restarts the
     * counter 
     */
    public function rewind(){
        $this->_count=0;
    }
    
    /**
     * @brief Returns the curren position
     * @return integer
     */
    public function key(){
        return $this->_count;
    }

    /**
     * @brief checks if the counter is less than the number of rows in
     * the current page
     * @return bool
     */    
    public function valid(){
        return $this->_count<$this->_page_length;
    }

    /**
     * @brief Following functino performs a query and executes a callback for each row
     * @param function   callback to be executed for each row
     * @param array      array with arguments for callback function
     */
    public function each($callback,&$args){
        $ret = null;
        if(is_object($this->_result)){
            while(($row=$this->_result->unbuffered_row()) && $ret!==FALSE){
                $data = json_decode($row->data,true);
                if($this->_class){
                    $class = $this->_class;
                    $obj = $class::model($this->_node);
                    $obj->set_data_values($data);
                    $row = $obj;
                }
                $ret = $callback($row,$args);
            }
        }
        return $ret;
    }


    public static function model(&$node,$class=null,$as_object=true){
        return new Safe_db_data_source_class($class,$node,$as_object);
    }
}