<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * File data source
 * @author Santiago Tramoyeres Cuesta <scuesta@sangoma.com>
 *         
 */
require_once(dirname(__FILE__).'/safe_data_source_interface.php');

/** 
 *  This class is an abstraction layer for directory reading. It allows
 *  filtering on files under one root directory. It also implements Iterator
 *  interface to use the instance of this class into a foreach loop.
 *  
 *  Example1: 
 *      $fds = new Safe_file_data_source('/home/user',true);
 *      foreach($fds->find(array('file'=>'filename.png')) as $i => $row)
 *        ...
 * 
 *  Example2: 
 *      $fds = new Safe_file_data_source('/home/user',true);
 *      foreach($fds->find(array('date'=>"between '2016-01-01' and '2016-01-31'")) as $i => $row)
 *        ...
 *
 *
 */
class Safe_file_data_source implements Safe_data_source_interface, Iterator {
    private $dpath;
    private $recursive;
    private $results;
    private $_position;
    private $_count;
    private $_length;
    private $_offset;
    private $_limit;

    /**
     * @brief Constructor method
     * @param string  root directory which is going to be used as file source
     * @param bool    set recursion to search root directory in depth
     */
    public function __construct($directorypath,$recursive=false){
        $this->dpath = $directorypath;
        $this->recursive = $recursive;
        $this->results = array();
        $this->_position = 0;
        $this->_count=0;
        $this->_length=null;
        $this->_offset=0;
        $this->_limit=null;
    }
    
    /**
     * @brief Field returned for each row iteration
     * @return array date (file date creation), path (fullpath), file (filename)
     */
    public function fields(){
        return array('date','path','file');
    }
    
    /**
     * @brief This method let us filter from file date cretion
     * @param string    Comparision operator. It allows use between date1 and date2
     * @param string    First date to be compared with
     * @param string    Second date to be compared with
     */
    private function match_date($relation,$date1,$date2){        
        switch($relation){
        case '<':
            return $date1<strtotime($date2);
            break;
        case '>':
            return $date1>strtotime($date2);
            break;
        case '<=':
            return $date1<=strtotime($date2);
            break;
        case '>=':
            return $date1>=strtotime($date2);
            break;
        case '=':
            return $date1==strtotime($date2);
            break;
        case 'between':
            $dates = explode(" and ",$date2);            
            return count($dates)>0 && $date1>=strtotime($dates[0]) && $date1<=strtotime($dates[1]);
            break;
        }
        return false;
    }
    
    /**
     * @brief matching method for general purpose
     * @param array    Filtering criteria. Each one points the key over the
     *                 match should be performed. For example: array(array('date'=>'between
     *                 date1 and date2'))
     * @param array    Row relative to current file. It has the same keys as
     *                 fields() method returns.
     */
    private function match($query,$row){
        $relations = array('<'=>true,'>'=>true,'<='=>true,'>='=>true,'='=>true,'between'=>true);
        $match=true;
        foreach($query as $key => $val){
            if($key=='date'){
                if(is_array($val) && isset($relations[$val[0]]) && count($val)>1){
                    $match = $match && $this->match_date($val[0],$row['date'],$val[1]);
                }
                else if(is_array($val))
                    $match = $match && $this->match_date('=',$row['date'],$val[0]);
                else
                    $match = $match && $this->match_date('=',$row['date'],$val);                    
            }else if($key=='path'){
                $match = $match && preg_match('#'.$val.'#i',$row['path']);
            }else if($key=='file'){
                $match = $match && preg_match('#'.$val.'#i',$row['file']);
            }
            else
                $match = false;
            if(!$match)
                break;
        }
        return $match;
    }    

    /**
     * @brief Main iterator. It reads the root directory and executes callback
     *        with the arguments passed as parametter and information from current file
     * @param function   callback to be executed for each file
     * @param array      array with arguments for callback function
     */
    public function each($callback,&$args=array()){
        $ret = null;
        if(is_dir($this->dpath) && !preg_match('#\.$#',$this->dpath)){
            $d = @opendir($this->dpath);
            while(($path=@readdir($d)) && $ret!==FALSE){
                $fullpath = $this->dpath.'/'.$path;
                $state = @stat($fullpath);
                if($state && !preg_match('#\.$#',$fullpath))
                    $ret = $callback(array('path'=>$fullpath,'date'=>$state['mtime'],'file'=>basename($fullpath)),$args);
            }
            @closedir($d);
        }
        return $ret;
    }

    /**
     * @brief auxiliary method to filter having into account the number of files
     * added and the file read counter
     * @param array   filter predicate
     * @param int     file readed counter
     * @param int     number of file added
     * @param array   array to save files matching filter predicate
     * @return Safe_file_data_source current object
     */
    public function _find($query=array(),&$nfiles,&$added,&$results=array()){
        $args = (object) array('nfiles'=>&$nfiles,'added'=>&$added,'results'=>&$results,'offset'=>$this->_offset,'limit'=>$this->_limit,'self'=>&$this,'query'=>&$query);
        $this->each(function($row,&$args){
            $fullpath = $row['path'];
            if($args->limit!==null && $args->limit<=$args->added)
                return FALSE;
            
            $match = $args->self->match($args->query,$row);
            if(is_dir($fullpath) && $args->self->recursive){
                $fds = new Safe_file_data_source($fullpath,$args->self->recursive);  
                $fds->limit($args->limit)->offset($args->offset);
                foreach($fds->_find($args->query,$args->nfiles,$args->added,$args->results) as $row2){
                    if($args->nfiles>=$args->offset && ($args->limit===null || $args->limit>$args->added)){
                        $args->results[] = $row2;
                        $args->added++;
                    }else
                        break;
                }
                unset($fds);
            }else if($args->nfiles>=$args->offset && ($args->limit===null || $args->limit>$args->added) && $match){
                $args->results[] = $row;
                $args->added++;
            }else if($match && ($args->limit===null || $args->limit>$args->added))
                $args->nfiles++;
        },$args);
        return $this;
    }
    
    /**
     * @breif Method to use for filtering 
     * @param array  filter predicate
     * @return Safe_file_data_source current object
     */
    public function find($query=array()){
        $this->results = array();
        $nfiles = 0;
        $added = 0;
        $this->_find($query,$nfiles,$added,$this->results);
        $this->_position = 0;
        $this->_count = $added;
        $this->_offset = 0;
        $this->_limit = null;
        return $this;
    }
    
    /**
     * @brief Getter/Setter for offset
     * @param integer 
     * @return Safe_file_data_source current object
     */
    public function offset($offset=FALSE){
        if($offset===FALSE)
            return $this->_offset;
        $this->_offset = $offset;
        return $this;
    }

    /**
     * @brief Getter/Setter for limit
     * @param integer
     * @return Safe_file_data_source current object
     */
    public function limit($limit=FALSE){
        if($limit===FALSE)
            return $this->_limit;
        $this->_limit = $limit;
        return $this;
    }
    
    /**
     * @brief Get the current matched file. This method is requird by Iterator
     * interface and let use Safe_file_data_source in foreach statement.
     * @return array   Array with those params from fields method.
     */
    public function current(){        
        return isset($this->results[$this->_position])?
            $this->results[$this->_position] : null;
    }
    
    /**
     * @brief Increases the pointer for iteration
     */
    public function next(){
        $this->_position++;
    }

    /**
     * @brief Returns the number of matched files for the current page
     * @return integer
     */
    public function count(){
        return $this->_count;
    }


    /**
     * @brief If query param is NOT empty, returns the number of matched files
     *        for whole domain. If query param is empty, returns the number of files
     *        under root directory.
     * @param array   filter predicate
     * @return integer
     */
    public function length($query=array()){
        if($query==array()){
            if($this->_length===null){
                $args = (object) array('length'=>0,'recursive'=>$this->recursive);
                $this->each(function($row,&$args){
                        $fullpath = $row['path'];
                        if(is_dir($fullpath) && $args->recursive){
                            $fds = new Safe_file_data_source($fullpath,$args->recursive);
                            $args->length+=$fds->length();
                            unset($fds);
                        }else
                            $args->length++;
                    },$args);
                $this->_length = $args->length;
            }
            return $this->_length;
        }else{
            $matches = 0;
            $args = (object) array('matches'=>&$matches,'self'=>$this,'query'=>&$query);
            $this->each(function($row,&$args){
                    if($args->self->match($args->query,$row))
                        $args->matches++;
                    if(is_dir($row['path']) && $args->self->recursive){
                        $fds = new Safe_file_data_source($row['path'],$args->self->recursive);
                        $args->matches+= $fds->length($args->query);
                        unset($fds);
                    }
                },$args);
            return $matches;
        }
    }
    
    /**
     * @brief Set the pointer to starting point
     */
    public function rewind(){
        $this->_position=0;
    }

    /**
     * @brief Returns the curren position
     * @return integer
     */
    public function key(){
        return $this->_position;
    }
    
    /**
     * @brief check if current position is valid to continue the iteration
     * @return bool
     */
    public function valid(){
        return !preg_match('#\.$#',$this->dpath) && $this->_position<$this->count();
    }
    
}

?>