<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: * */
/** coding: utf-8: * */
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * William Adam <wadam@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/**
 * SNG CWMP (TR-069) Configuration
 *
 * @author William Adam
 * @version
 */
if (!defined('BASEPATH'))
    exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');

class Sng_cwmp_config_class extends Safe_configurable_object_class {
    
    private $_script_path = '';
    const DEBUG_FILE = "/cwmp/etc/debug";
    const CONFIG_FILE = "/cwmp/etc/management.json";

    public function __construct($node, $parent_name) {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, "configuration", $node);
        $this->_script_path = $this->node()->configuration_manager()->directory('sng-base');
    }

    public function add_enum_helper($name, $label, $type, $value, $extra, $helptext = NULL) {
        if (is_bool($value)) {
            $value = $value ? "true" : "false";
        };
        $this->add_enum_field($name, $label, $type, "".$value, $extra);
        if ($helptext !== NULL) {
            $this->set_field_help($name, $helptext);
        };
    }

    public function add_item_helper($name, $label, $type, $value, $extra, $helptext = NULL) {
        $this->add_field($name, $label, $type, "".$value, $extra);
        if ($helptext !== NULL) {
            $this->set_field_help($name, $helptext);
        };
    }

    public function configure() {
        $data = json_decode(file_get_contents($this->_script_path . Sng_cwmp_config_class::CONFIG_FILE), true);
        $this->add_enum_helper('config/use_dhcp_acs', 'Use DHCP option 43', 'dropdown', $data['use_dhcp_acs'], $this->enable_disable_enum(),
            'Enable encapsulated CWMP parameters via DHCP option 43.');

        $this->add_item_helper('URL', 'ACS URL', 'text', $data['URL'], 90,
            'Currently configured ACS URL (read only) if DHCP option 43 is enabled, or static ACS URL if DHCP option 43 is disabled.');

        $this->add_enum_helper('PeriodicInformEnable', 'Periodic INFORM', 'dropdown', $data['PeriodicInformEnable'], $this->enable_disable_enum());
        $this->add_item_helper('PeriodicInformInterval', 'INFORM Interval', 'text', $data['PeriodicInformInterval'], '10');

        $this->add_item_helper('Username', 'ACS Username', 'text', $data['Username'], 30);
        $this->add_item_helper('Password', 'ACS Password', 'password', $data['Password'], 30);

        $this->add_enum_helper('config/allow_acs_only', 'Restrict connection to CWMP service', 'dropdown', $data['allow_acs_only'],
            $this->enable_disable_enum(), 'Allow connections to CWMP service only from configured ACS server IP address.');

        $this->add_item_helper('ConnectionRequestUsername', 'Connection Request Username', 'text', $data['ConnectionRequestUsername'], 30,
            'Username for authenticating ACS server connection to CPE.<br>NOTE: Requires a CWMP service restart to become effective.');

        $this->add_item_helper('ConnectionRequestPassword', 'Connection Request Password', 'password', $data['ConnectionRequestPassword'], 30,
            'Password for authenticating ACS server connection to CPE.<br>NOTE: Requires a CWMP service restart to become effective.');

        $this->add_item_helper('UDPConnectionRequestAddress', 'Connection Request Address and Port', 'text', $data['UDPConnectionRequestAddress'], 30,
            "Specify the address and port in format \"address:port\" where the ACS can contact the CPE directly.<br>NOTE: Requires CWMP service restart to become efffective.");

        $this->add_item_helper('ConnectionRequestURL', 'Connection Request URL', 'text', $data['ConnectionRequestURL'], 50);
        $this->set_field_readonly('ConnectionRequestURL');

        $this->add_item_helper('config/default_url', 'Default ACS URL', 'text', $data['default_url'], 90,
            "Default ACS URL if no DHCP option 43 is received, only available if DHCP option 43 is enabled.<br>NOTE: Requires a full network restart to become effective.");

        $this->add_enum_helper('config/default_inform_enabled', 'Default periodic INFORM', 'dropdown', $data['default_inform_enabled'],
            $this->enable_disable_enum(), 'Default option used for Periodic INFORM if no DHCP option 43 is received.');

        $this->add_item_helper('config/default_inform_interval', 'Default INFORM interval', 'text', $data['default_inform_interval'],
            '10', 'Default option used for Periodic INFORM interval if no DHCP option 43 is received.');

        $this->add_enum_helper('config/auto_reload', 'Restart or reload when configured', 'dropdown', $data['auto_reload'], $this->enable_disable_enum(),
            'Automatically reload or restart NSC service when configuration is changed via ACS server');

        $this->add_enum_helper('config/debug', 'Enable debug mode for services', 'dropdown', file_exists($this->_script_path . Sng_cwmp_config_class::DEBUG_FILE) ? "true" : "false", $this->enable_disable_enum(),
            'Enable debug mode for CWMP services, should not be used naively on production systems.<br>NOTE: Only for debugging CWMP services - for debugging TR-069 protocol, please use packet capturing facilities.');

        $control_group_fields = array('URL', 'PeriodicInformEnabled', 'PeriodicInformInterval',
            'config/default_url', 'config/default_inform_enabled', 'config/default_inform_interval');

        foreach ($control_group_fields as $group_key) {
            if (strpos($group_key, 'config/default') !== 0) {
                $condition_data['true'][] = array('field_name' => $group_key, 'status' => 'disable');
                $condition_data['false'][] = array('field_name' => $group_key, 'status' => 'enable');
            } else {
                $condition_data['true'][] = array('field_name' => $group_key, 'status' => 'enable');
                $condition_data['false'][] = array('field_name' => $group_key, 'status' => 'disable');
            }
        }

        $this->create_group('automatic', $control_group_fields,
            array('default' => array('value' => 'true', 'status' => 'enable'), 'condition' => $condition_data)
        );

        $this->conditional_control('config/use_dhcp_acs', 'automatic');

        $this->set_field_category('config/use_dhcp_acs', 'Connection');
        $this->set_field_category('config/default_url', 'Fallback connection');
        $this->set_field_category('ConnectionRequestUsername', 'Connection request');
        $this->set_field_category('config/auto_reload', 'General');

        return parent::configure();
    }

    public function save(&$output = null)
    {
        $ret = parent::save($output);

        $data = array();
        $excludes = array('ConnectionRequestURL');
        $numerics = array('default_inform_interval', 'PeriodicInformInterval');
        $booleans = array('use_acs_dhcp', 'allow_acs_only', 'auto_reload', 'debug',
            'default_inform_enabled', 'PeriodicInformEnable');

        $cfg = $this->get_data_values(false);

        foreach($cfg as $item => $value){
            $key = $item;
            if (in_array($key, $excludes)) {
                continue;
            };
            if (strpos($item, 'config/') === 0) {
                $key = substr($item, 7);
            };
            if (in_array($key, $numerics)) {
                $value = intval($value);
            }
            else if (in_array($key, $booleans)) {
                $value = ($value === 'true');
            };
            $data[$key] = $value;
        }

        $jsondata = json_encode($data);
        safe_exec("{$this->_script_path}/bin/sng-cwmp-client --string ConfigUpdate '" . $jsondata . "'", '', '', $rc, false);
        return $ret;
    }

    protected function _set_unserialized_data($data)
    {
        // Never cache current values
        return TRUE;
    }
}
